import unittest
from cStringIO import StringIO
import sys
import turbogears
from turbogears import controllers
from turbogears import url
from turbogears import validators
from turbogears import database
from turbogears import testutil
import cherrypy
import formencode
import pkg_resources

class SubApp(controllers.RootController):
    def index(self):
        return url("/Foo/")
    index = turbogears.expose()(index)

class MyRoot(controllers.RootController):
    def index(self):
        pass
    index = turbogears.expose()(index)

    def validation_error_handler(self, tg_source, tg_errors, *args, **kw):
        self.functionname = tg_source.__name__
        self.values = kw
        self.errors = tg_errors
        return "Error Message"

    def test(self):
        return dict(title="Foobar", mybool=False, someval="niggles")
    test = turbogears.expose(html=".simple",
            allow_json=True)(test)

    def invalid(self):
        return None
    invalid = turbogears.expose()(invalid)

    def pos(self, posvalue):
        self.posvalue = posvalue
        return ""
    pos = turbogears.expose()(pos)

    def servefile(self, tg_exceptions=None):
        self.servedit = True
        self.serve_exceptions = tg_exceptions
        return cherrypy.lib.cptools.serveFile(
            pkg_resources.resource_filename(
                "turbogears.tests", "test_controllers.py"))
    servefile = turbogears.expose()(servefile)

    def unicode(self):
        cherrypy.response.headers["Content-Type"] = "text/html"
        return u'\u00bfHabla espa\u00f1ol?'
    unicode = turbogears.expose()(unicode)

    def returnedtemplate(self):
        return dict(title="Foobar", mybool=False, someval="foo",
            tg_template="turbogears.tests.simple")
    returnedtemplate = turbogears.expose()(returnedtemplate)

    def returnedtemplate_short(self):
        return dict(title="Foobar", mybool=False, someval="foo",
            tg_template=".simple")
    returnedtemplate_short = turbogears.expose()(returnedtemplate_short)

    def exposetemplate_short(self):
        return dict(title="Foobar", mybool=False, someval="foo")
    exposetemplate_short = turbogears.expose(html=".simple")(exposetemplate_short)

    def istrue(self, value):
        self.value = value
        return str(value)
    istrue = turbogears.error_handler(validation_error_handler)(istrue)
    istrue = turbogears.validate(validators={
                'value': validators.StringBoolean()})(istrue)
    istrue = turbogears.expose()(istrue)

    def callsanother(self):
        return self.istrue(True)
    callsanother = turbogears.expose()(callsanother)

    def returnjson(self):
        return dict(title="Foobar", mybool=False, someval="foo",
            tg_html="turbogears.tests.simple")
    returnjson = turbogears.expose(format="json",
            html="turbogears.tests.simple")(returnjson)

    def allowjson(self):
        return dict(title="Foobar", mybool=False, someval="foo",
             tg_html="turbogears.tests.simple")
    allowjson = turbogears.expose(html="turbogears.tests.simple",
            allow_json=False)(allowjson)

    def impliedjson(self):
        return dict(title="Blah")
    impliedjson = turbogears.expose(format="json")(impliedjson)

    def contenttype(self):
        return "Foobar"
    contenttype = turbogears.expose(content_type="xml/atom")(contenttype)

    def save(self, submit, firstname, lastname="Miller"):
        self.submit = submit
        self.firstname = firstname
        self.lastname = lastname
        self.fullname = "%s %s" % (self.firstname, self.lastname)
        return self.fullname
    save = turbogears.error_handler(validation_error_handler)(save)
    save = turbogears.validate(validators={
        "firstname": validators.String(min=2, not_empty=True),
        "lastname": validators.String()})(save)
    save = turbogears.expose()(save)

    class Registration(formencode.Schema):
        allow_extra_fields = True
        firstname = validators.String(min=2, not_empty=True)
        lastname = validators.String()

    def save2(self, submit, firstname, lastname="Miller"):
        return self.save(submit, firstname, lastname)
    save2 = turbogears.error_handler(validation_error_handler)(save2)
    save2 = turbogears.validate(validators=Registration())(save2)
    save2 = turbogears.expose()(save2)

    def useother(self):
        return dict(tg_template="turbogears.tests.othertemplate")
    useother = turbogears.expose(html="turbogears.tests.simple")(useother)

    rwt_called = 0
    def rwt(self, func, *args, **kw):
        self.rwt_called += 1
        func(*args, **kw)

    def flash_plain(self):
        turbogears.flash("plain")
        return dict(title="Foobar", mybool=False, someval="niggles")
    flash_plain = turbogears.expose(html=".simple",
        allow_json=True)(flash_plain)

    def flash_unicode(self):
        turbogears.flash(u"\xfcnicode")
        return dict(title="Foobar", mybool=False, someval="niggles")
    flash_unicode = turbogears.expose(html=".simple",
        allow_json=True)(flash_unicode)

    def flash_data_structure(self):
        turbogears.flash(dict(uni=u"\xfcnicode", testing=[1, 2, 3]))
        return dict(title="Foobar", mybool=False, someval="niggles")
    flash_data_structure = turbogears.expose(html=".simple",
        allow_json=True)(flash_data_structure)

    def flash_redirect(self):
        turbogears.flash(u"redirect \xfcnicode")
        turbogears.redirect("/flash_redirected?tg_format=json")
    flash_redirect = turbogears.expose(html=".simple",
        allow_json=True)(flash_redirect)

    def flash_redirected(self):
        return dict(title="Foobar", mybool=False, someval="niggles")
    flash_redirected = turbogears.expose(html=".simple",
        allow_json=True)(flash_redirected)


class TestRoot(unittest.TestCase):
    def setUp(self):
        cherrypy.root = None
        cherrypy.tree.mount_points = {}
        cherrypy.tree.mount(MyRoot(), "/")
        cherrypy.tree.mount(SubApp(), "/subthing")

    def tearDown(self):
        cherrypy.root = None
        cherrypy.tree.mount_points = {}

    def test_jsFiles(self):
        'Can access the JavaScript files'
        testutil.createRequest("/tg_js/MochiKit.js")
        self.failUnlessEqual("application/x-javascript",
            cherrypy.response.headers["Content-Type"])
        self.failUnlessEqual("200 OK", cherrypy.response.status)

    def test_jsonOutput(self):
        testutil.createRequest("/test?tg_format=json")
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values == dict(title="Foobar", mybool=False, someval="niggles",
            tg_flash=None)
        assert cherrypy.response.headers["Content-Type"] == "text/javascript"

    def test_impliedJson(self):
        testutil.createRequest("/impliedjson?tg_format=json")
        assert '"title": "Blah"' in cherrypy.response.body[0]

    def test_allowJson(self):
        testutil.createRequest("/allowjson?tg_format=json")
        assert cherrypy.response.headers["Content-Type"] == "text/html"

    def test_allowJsonConfig(self):
        "JSON output can be enabled via config."
        turbogears.config.update({'tg.allow_json':True})
        testutil.capture_log("tubrogears.controllers")
        class JSONRoot(controllers.RootController):
            def allowjsonconfig(self):
                return dict(title="Foobar", mybool=False, someval="foo",
                     tg_html="turbogears.tests.simple")
            allowjsonconfig = turbogears.expose(html="turbogears.tests.simple")(allowjsonconfig)
        testutil.print_log()
        cherrypy.root = JSONRoot()
        testutil.createRequest('/allowjsonconfig?tg_format=json')
        assert cherrypy.response.headers["Content-Type"]=="text/javascript"
        turbogears.config.update({'tg.allow_json':False})

    def test_allowJsonConfigFalse(self):
        "Make sure JSON can still be restricted with a global config on."
        turbogears.config.update({'tg.allow_json':True})
        testutil.capture_log("tubrogears.controllers")
        class JSONRoot(controllers.RootController):
            def allowjsonconfig(self):
                return dict(title="Foobar", mybool=False, someval="foo",
                     tg_html="turbogears.tests.simple")
            allowjsonconfig = turbogears.expose(html="turbogears.tests.simple")(allowjsonconfig)
        testutil.print_log()
        cherrypy.root = JSONRoot()
        testutil.createRequest('/allowjson?tg_format=json')
        print cherrypy.response.body[0]
        assert cherrypy.response.headers["Content-Type"]=="text/html"
        turbogears.config.update({'tg.allow_json':False})

    def test_invalidreturn(self):
        testutil.create_request("/invalid")
        print cherrypy.response.status
        assert cherrypy.response.status.startswith("500")

    def test_strict_parameters(self):
        turbogears.config.update({"tg.strict_parameters" : True})
        testutil.create_request("/save?submit=save&firstname=Foo&lastname=Bar&badparam=1")
        print cherrypy.response.status
        assert cherrypy.response.status.startswith("500")
        assert not hasattr(cherrypy.root, "errors")

    def test_retrieveDictDirectly(self):
        d = testutil.call(cherrypy.root.returnjson)
        assert d["title"] == "Foobar"

    def test_templateOutput(self):
        testutil.createRequest("/test")
        assert "Paging all niggles" in cherrypy.response.body[0]

    def test_throw_out_random(self):
        """A random value can be appended to the URL to avoid caching
        problems."""
        testutil.createRequest("/test?tg_random=1")
        assert "Paging all niggles" in cherrypy.response.body[0]

    def test_safariUnicodeFix(self):
        testutil.createRequest("/unicode", headers={'User-Agent' :
            "Apple WebKit Safari/412.2"})
        firstline = cherrypy.response.body[0].split('\n')[0]
        self.failUnlessEqual("&#xbf;Habla espa&#xf1;ol?", firstline)
        self.failUnless(isinstance(firstline, str))

    def test_defaultFormat(self):
        """The default format can be set via expose"""
        testutil.createRequest("/returnjson")
        firstline = cherrypy.response.body[0]
        assert '"title": "Foobar"' in firstline
        testutil.createRequest("/returnjson?tg_format=html")
        firstline = cherrypy.response.body[0]
        assert '"title": "Foobar"' not in firstline

    def test_contentType(self):
        """The content-type can be set via expose"""
        testutil.createRequest("/contenttype")
        assert cherrypy.response.headers["Content-Type"] == "xml/atom"

    def test_returnedTemplateName(self):
        testutil.createRequest("/returnedtemplate")
        data = cherrypy.response.body[0].lower()
        assert "<body>" in data
        assert 'groovy test template' in data

    def test_returnedTemplateShort(self):
        testutil.createRequest("/returnedtemplate_short")
        assert "Paging all foo" in cherrypy.response.body[0]

    def test_exposeTemplateShort(self):
        testutil.createRequest("/exposetemplate_short")
        assert "Paging all foo" in cherrypy.response.body[0]

    def test_validation(self):
        "Data can be converted and validated"
        testutil.createRequest("/istrue?value=true")
        assert cherrypy.root.value is True
        testutil.createRequest("/istrue?value=false")
        assert cherrypy.root.value is False
        cherrypy.root = MyRoot()
        testutil.createRequest("/istrue?value=foo")
        assert not hasattr(cherrypy.root, "value")
        assert cherrypy.root.functionname == "istrue"

        testutil.createRequest("/save?submit=send&firstname=John&lastname=Doe")
        assert cherrypy.root.fullname == "John Doe"
        assert cherrypy.root.submit == "send"
        testutil.createRequest("/save?submit=send&firstname=Arthur")
        assert cherrypy.root.fullname == "Arthur Miller"
        testutil.createRequest("/save?submit=send&firstname=Arthur&lastname=")
        assert cherrypy.root.fullname == "Arthur "
        testutil.createRequest("/save?submit=send&firstname=D&lastname=")
        assert len(cherrypy.root.errors) == 1
        assert cherrypy.root.errors.has_key("firstname")
        assert "characters" in cherrypy.root.errors["firstname"].msg.lower()
        testutil.createRequest("/save?submit=send&firstname=&lastname=")
        assert len(cherrypy.root.errors) == 1
        assert cherrypy.root.errors.has_key("firstname")

    def test_validationwithschema(self):
        "Data can be converted and validated with formencode.Schema instance"
        testutil.createRequest("/save2?submit=send&firstname=John&lastname=Doe")
        assert cherrypy.root.fullname == "John Doe"
        assert cherrypy.root.submit == "send"
        testutil.createRequest("/save2?submit=send&firstname=Arthur&lastname=")
        assert cherrypy.root.fullname == "Arthur "
        testutil.createRequest("/save2?submit=send&firstname=&lastname=")
        assert len(cherrypy.root.errors) == 1
        assert cherrypy.root.errors.has_key("firstname")
        testutil.createRequest("/save2?submit=send&firstname=D&lastname=")
        assert len(cherrypy.root.errors) == 1
        assert cherrypy.root.errors.has_key("firstname")

    def test_othertemplate(self):
        "'tg_html' in a returned dict will use the template specified there"
        testutil.createRequest("/useother")
        assert "This is the other template" in cherrypy.response.body[0]

    def test_runwithtrans(self):
        "run_with_transaction is called only on topmost exposed method"
        oldrwt = database.run_with_transaction
        database.run_with_transaction = cherrypy.root.rwt
        testutil.createRequest("/callsanother")
        database.run_with_transaction = oldrwt
        assert cherrypy.root.value
        assert cherrypy.root.rwt_called == 1

    def test_positional(self):
        "Positional parameters should work"
        testutil.createRequest("/pos/foo")
        assert cherrypy.root.posvalue == "foo"

    def test_flash_plain(self):
        "turbogears.flash with strings should work"
        testutil.createRequest("/flash_plain?tg_format=json")
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"]=="plain"
        assert not cherrypy.response.simple_cookie.has_key("tg_flash")

    def test_flash_unicode(self):
        "turbogears.flash with unicode objects should work"
        testutil.createRequest("/flash_unicode?tg_format=json")
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"]==u"\xfcnicode"
        assert not cherrypy.response.simple_cookie.has_key("tg_flash")

    def test_flash_on_redirect(self):
        "turbogears.flash must survive a redirect"
        testutil.createRequest("/flash_redirect?tg_format=json")
        assert cherrypy.response.status.startswith("302")
        testutil.createRequest(
            cherrypy.response.headers["Location"],
            headers=dict(Cookie=cherrypy.response.simple_cookie.output(header="").strip()))
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"]==u"redirect \xfcnicode"

    def test_double_flash(self):
        """latest set flash should have precedence"""
        # Here we are calling method that sets a flash message. However flash
        # cookie is still there. Turbogears should discard old flash message
        # from cookie and use new one, set by flash_plain().
        testutil.createRequest("/flash_plain?tg_format=json",
                               headers=dict(Cookie='tg_flash="old flash"; Path=/;'))
        import simplejson
        values = simplejson.loads(cherrypy.response.body[0])
        assert values["tg_flash"]=="plain"
        assert cherrypy.response.simple_cookie.has_key("tg_flash"), \
                "Cookie clearing request should be present"
        flashcookie = cherrypy.response.simple_cookie['tg_flash']
        assert flashcookie['expires'] == 0

    def test_set_kid_outputformat_in_config(self):
        "the outputformat for kid can be set in the config"
        turbogears.config.update({'kid.outputformat': 'xhtml'})
        testutil.createRequest('/test')
        assert '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML ' in cherrypy.response.body[0]
        turbogears.config.update({'kid.outputformat': 'html'})
        testutil.createRequest('/test')
        assert  '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML ' in cherrypy.response.body[0]

    def test_fileserving(self):
        #outputcap = StringIO()
        #sys.stdout = outputcap
        testutil.create_request("/servefile")
        assert cherrypy.root.servedit
        assert not cherrypy.root.serve_exceptions
        #assert "AssertionError" not in outputcap.getvalue()

class TestURLs(unittest.TestCase):
    def setUp(self):
        cherrypy.tree.mount_points = {}
        cherrypy.root = MyRoot()
        cherrypy.root.subthing = SubApp()
        cherrypy.root.subthing.subsubthing = SubApp()

    def test_basicurls(self):
        testutil.createRequest("/")
        self.failUnlessEqual("/foo", url("/foo"))
        self.failUnlessEqual("foo/bar", url(["foo", "bar"]))
        assert url("/foo", bar=1, baz=2) in \
                ["/foo?bar=1&baz=2", "/foo?baz=2&bar=1"]
        assert url("/foo", dict(bar=1, baz=2)) in \
                ["/foo?bar=1&baz=2", "/foo?baz=2&bar=1"]
        assert url("/foo", dict(bar=1, baz=None)) == "/foo?bar=1"

    def test_url_without_request_available(self):
        cherrypy.serving.request = None
        self.assertEquals("/foo", turbogears.url("/foo"))

    def test_approots(self):
        testutil.createRequest("/subthing/")
        self.failUnlessEqual("foo", url("foo"))
        self.failUnlessEqual("/subthing/foo", url("/foo"))

    def test_lowerapproots(self):
        testutil.create_request("/subthing/subsubthing/")
        print url("/foo")
        assert "/subthing/subsubthing/foo" == url("/foo")

    def test_approotsWithPath(self):
        turbogears.config.update({"server.webpath" : "/coolsite/root"})
        turbogears.startup.startTurboGears()
        testutil.createRequest("/coolsite/root/subthing/")
        print cherrypy.tree.mount_point()
        self.failUnlessEqual("/coolsite/root/subthing/foo",
                        url("/foo"))

    def test_redirect(self):
        turbogears.config.update({"server.webpath" : "/coolsite/root"})
        turbogears.startup.startTurboGears()
        testutil.createRequest("/coolsite/root/subthing/")
        try:
            turbogears.redirect("/foo")
            assert False, "redirect exception should have been raised"
        except cherrypy.HTTPRedirect, e:
            print e.urls
            assert "http://localhost/coolsite/root/subthing/foo" in e.urls

        try:
            raise turbogears.redirect("/foo")
            assert False, "redirect exception should have been raised"
        except cherrypy.HTTPRedirect, e:
            print e.urls
            assert "http://localhost/coolsite/root/subthing/foo" in e.urls


    def tearDown(self):
        turbogears.config.update({"server.webpath" : ""})
        turbogears.startup.startTurboGears()

def test_index_trailing_slash():
    "If there is no trailing slash on an index method call, redirect"
    cherrypy.root = SubApp()
    cherrypy.root.foo = SubApp()
    testutil.createRequest("/foo")
    print cherrypy.response.status
    assert cherrypy.response.status.startswith("302")

def test_can_use_internally_defined_arguments():
    """Tests that we can use argument names that are internally used by TG
    in controllers:_execute_func et al."""
    class App(controllers.RootController):
        def index(self, **kw):
            return "\n".join(["%s:%s" % i for i in kw.iteritems()])
        index = turbogears.expose()(index)

    cherrypy.root = App()
    testutil.createRequest("/?format=foo&template=bar&fragment=boo")
    output = cherrypy.response.body[0]
    print output
    assert "format:foo" in output
    assert "template:bar" in output
    assert "fragment:boo" in output



"""
SQLObject-based version of gettext
"""

import turbogears

from turbogears.i18n.sogettext.model import TG_Message, TG_Domain
from turbogears.i18n.utils import get_locale

try:
    from sqlobject import SQLObjectNotFound
except ImportError:
    pass
from gettext import translation

import codecs

_catalogs = {}

def so_gettext(key, locale=None, domain=None):
    """SQLObject-based version of gettext. Messages are stored in database instead.
    """
    
    locale = get_locale(locale)
    
    messages = get_so_catalog(domain).get(locale)
    if not messages:
        messages = get_so_catalog(domain).get(locale[:2], {})
        
    return unicode(messages.get(key, key))

def get_so_catalog(domain):
    """Retrieves all translations for locale and domain from database and stores in thread data.
    """

    if domain is None:domain = turbogears.config.get("i18n.domain", "messages")
    
    catalog = _catalogs.get(domain)
    if not catalog:
        
        catalog = {}
        
        try:
            domain = TG_Domain.byName(domain)
        except SQLObjectNotFound:
            return catalog
            
        results = TG_Message.selectBy(domain=domain)
        for message in results:
            locale = message.locale    
            messages = catalog.get(locale, {})
            messages[message.name] = message.text
            catalog[locale] = messages
        
        
        _catalogs[domain.name] = catalog
            
    return catalog

def create_so_catalog(locales, domain):
    """Creates a message catalog based on list of locales from existing GNU message catalog
    """

    TG_Message.dropTable(ifExists=True)
    TG_Domain.dropTable(ifExists=True)

    TG_Domain.createTable(ifNotExists=True)
    TG_Message.createTable(ifNotExists=True)
    localedir = turbogears.config.get("i18n.locale_dir", "locales")
        
    try:
        domain = TG_Domain.byName(domain)
        return
    except SQLObjectNotFound:
        domain = TG_Domain(name=domain)
        
    for locale in locales:
        translations = translation(domain=domain.name, localedir=localedir, languages=[locale])
        catalog = translations._catalog
        for k, v in catalog.items():
        
            TG_Message(domain=domain, locale=locale, name=k, text=v) 
 
def dump_so_catalogs(locales):
    """Takes all domains and messages and creates message catalogs 
    """
    localedir = turbogears.config.get("i18n.locale_dir", "locales")

    for locale in locales:
    
        messages_dir = os.path.join(localedir, locale, "LC_MESSAGES")
        
        for domain in TG_Domain.select():
            pofile=os.path.join(messages_dir, "%s.po" %domain.name)
            f = codecs.open(pofile, "w", "UTF-8")
            f.write("""
# SOME DESCRIPTIVE TITLE.
# Copyright (C) YEAR ORGANIZATION
# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
#
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"POT-Creation-Date: CREATION_DATE\\n"
"PO-Revision-Date: REVISION_DATE\\n"
"Last-Translator: TRANSLATOR <EMAIL@ADDRESS>\\n"
"Language-Team: LANGUAGE <LL@li.org>\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=UTF-8\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Generated-By: turbogears\\n"
            
            """)
            
            for message in TG_Message.selectBy(domain=domain, locale=locale):
            
                if message.name == "":continue # descriptive text
                
                f.write(u"""
msgid "%s"
msgstr "%s"
                """%(message.name, message.text))
            
            f.close()



import logging
import sha
import threading
from random import random
from datetime import timedelta, datetime

import cherrypy
import pkg_resources
from cherrypy.filters.basefilter import BaseFilter

import turbogears


log = logging.getLogger("turbogears.visit")

# Global VisitManager
_manager= None

# Global list of plugins for the Visit Tracking framework
_plugins= []

# Accessor functions for getting and setting the current visit information.
def current():
    '''
    Retrieve the current visit record.
    '''
    return getattr( cherrypy.request, "tg_visit", None )

def set_current( visit ):
    '''
    Set the current visit record.
    '''
    cherrypy.request.tg_visit= visit


def _create_visit_manager(timeout):
    '''
    Create a VisitManager based on the plugin specified in the config file.
    '''
    plugin_name = turbogears.config.get( "visit.manager", "sqlobject" )
    plugins= pkg_resources.iter_entry_points( "turbogears.visit.manager",
                                              plugin_name )

    log.debug( "Loading visit manager from plugin: %s", plugin_name )

    for entrypoint in plugins:
        plugin= entrypoint.load()
        return plugin( timeout )

    raise RuntimeError( "VisitManager plugin missing: %s" % plugin_name )

# Interface for the TurboGears extension
def start_extension():
    # Bail out if the application hasn't enabled this extension
    if not turbogears.config.get( "visit.on", False ):
        return
    # Bail out if this extension is already running
    global _manager
    if _manager:
        return
    log.info( "Visit Tracking starting" )
    # How long may the visit be idle before a new visit ID is assigned?
    # The default is 20 minutes.
    timeout= timedelta( minutes=turbogears.config.get( "visit.timeout", 20 ) )
    # Create the thread that manages updating the visits
    _manager= _create_visit_manager( timeout )

    filter= VisitFilter()
    # Temporary until tg-admin can call create_extension_model
    create_extension_model()
    # Install Filter into the root filter chain
    if not hasattr(cherrypy.root, "_cp_filters"):
        cherrypy.root._cp_filters= []
    cherrypy.root._cp_filters.append( filter )


def shutdown_extension():
    # Bail out if this extension is not running
    global _manager
    if not _manager:
        return
    log.info( "Visit Tracking shutting down" )
    _manager.shutdown()
    _manager = None


def create_extension_model():
    # Create the data model of the VisitManager if one exists.
    if _manager:
        _manager.create_model()


def enable_visit_plugin( plugin ):
    '''
    Register a visit tracking plugin. These plugins will be called for each
    request.
    '''
    _plugins.append( plugin )


class Visit(object):
    '''
    Basic container for visit related data.
    '''
    def __init__( self, key, is_new ):
        self.key= key
        self.is_new= is_new


class VisitFilter(BaseFilter):
    '''
    A filter that automatically tracks visitors.
    '''

    def __init__( self ):
        log.info( "Visit filter initialised" )
        get=turbogears.config.get

        # Get the name to use for the identity cookie.
        self.cookie_name = get( "visit.cookie.name", "tg-visit" )
        # The path should probably default to whatever the root is masquerading
        # as in the event of a virtual path filter.
        self.cookie_path= get( "visit.cookie.path", "/" )
        # The secure bit should be set for HTTPS only sites
        self.cookie_secure= get( "visit.cookie.secure", False )
        # By default, I don't specify the cookie domain.
        self.cookie_domain= get( "visit.cookie.domain", None )
        assert self.cookie_domain!="localhost", \
               "localhost is not a valid value for visit.cookie.domain. Try None instead."

    def before_main(self):
        '''
        Inspect the submitted request to determine whether it already belongs to
        an existing visit.
        '''
        if not turbogears.config.get( "visit.on", True ):
            set_current( None )
            return

        visit= None
        cookies= cherrypy.request.simple_cookie
        if self.cookie_name in cookies:
            # Process visit based on cookie
            visit_key= cookies[self.cookie_name].value
            visit= _manager.visit_for_key( visit_key )

        if not visit:
            visit_key=self._generate_key()
            visit= _manager.new_visit_with_key( visit_key )
            self.send_cookie( visit_key )

        set_current( visit )

        # Inform all the plugins that a request has been made for the current
        # visit. This gives plugins the opportunity to track click-path or
        # retrieve the visitor's identity.
        try:
            for plugin in _plugins:
                plugin.record_request( visit )
        except cherrypy.InternalRedirect, e:
            # Can't allow an InternalRedirect here because CherryPy is dumb,
            # instead change cherrypy.request.object_path to the url desired.
            cherrypy.request.object_path= e.path

    def _generate_key(self):
        '''
        Returns a (pseudo)random hash based on seed
        '''
        # Adding remoteHost and remotePort doesn't make this any more secure,
        # but it makes people feel secure... It's not like I check to make
        # certain you're actually making requests from that host and port. So
        # it's basically more noise.
        key_string= '%s%s%s%s' % (random(), datetime.now(),
                                  cherrypy.request.remote_host,
                                  cherrypy.request.remote_port)
        return sha.new(key_string).hexdigest()

    def clear_cookie( self ):
        '''
        Clear any existing visit ID cookie.
        '''
        cookies= cherrypy.response.simple_cookie

        # clear the cookie
        log.debug( "Clearing visit ID cookie" )
        cookies[self.cookie_name]= ''
        cookies[self.cookie_name]['path']= self.cookie_path
        cookies[self.cookie_name]['expires']= 0

    def send_cookie( self, visit_key ):
        '''
        Send an visit ID cookie back to the browser.
        '''
        cookies= cherrypy.response.simple_cookie
        cookies[self.cookie_name]= visit_key
        cookies[self.cookie_name]['path']= self.cookie_path
        if self.cookie_secure:
            cookies[self.cookie_name]['secure']= True
        if self.cookie_domain:
            cookies[self.cookie_name]['domain']= self.cookie_domain
        log.debug( "Sending visit ID cookie: %s",
                   cookies[self.cookie_name].output() )


class BaseVisitManager(threading.Thread):
    def __init__(self, timeout):
        super(BaseVisitManager,self).__init__( name="VisitManager" )
        self.timeout= timeout
        self.queue= dict()
        self.lock= threading.Lock()
        self._shutdown= threading.Event()
        self.interval= 30
        self.setDaemon(True)
        self.start()

    def create_model(self):
        pass

    def new_visit_with_key(self, visit_key):
        '''
        Return a new Visit object with the given key.
        '''
        raise NotImplementedError

    def visit_for_key(self, visit_key):
        '''
        Return the visit for this key or None if the visit doesn't exist or has
        expired.
        '''
        raise NotImplementedError

    def update_queued_visits(self, queue):
        '''
        Extend the expiration of the queued visits.
        '''
        raise NotImplementedError

    def update_visit(self, visit_key, expiry):
        try:
            self.lock.acquire()
            self.queue[visit_key]= expiry
        finally:
            self.lock.release()

    def shutdown(self, timeout=None):
        self._shutdown.set()
        self.join( timeout )
        if self.isAlive():
            log.error( "Visit Manager thread failed to shutdown." )

    def run(self):
        while not self._shutdown.isSet():
            self.lock.acquire()
            queue = None
            try:
                # make a copy of the queue and empty the original
                if self.queue:
                    queue = self.queue.copy()
                    self.queue.clear()
            finally:
                self.lock.release()
            if queue is not None:
                self.update_queued_visits(queue)
            self._shutdown.wait(self.interval)





#!/usr/bin/env python
import pkg_resources
pkg_resources.require("TurboGears")

import cherrypy
from os.path import *
import sys

from turbogears import toolbox

root = toolbox.Toolbox()
cherrypy.root = root
print "Toolbox Development Mode"
cherrypy.config.update({"global" : {
    "server.socketPort" : 7654,
    "server.environment" : "development",
    "server.logToScreen" : True,
    "autoreload.package" : "turbogears",
    "logDebugInfoFilter.on" : False
    }})

cherrypy.server.start()



import itertools
from datetime import datetime

from turbogears import validators, expose
from turbojson import jsonify
from turbogears.widgets.base import CSSLink, JSLink, CSSSource, JSSource, \
                                    Widget, WidgetsList, static, mochikit, \
                                    CoreWD
from turbogears.widgets.i18n import CalendarLangFileLink
from turbogears.widgets.forms import FormField, CompoundFormField, TextField, \
                                    HiddenField, TableForm, CheckBox, \
                                    RadioButtonList
from turbogears.widgets.rpc import RPC

__all__ = ["CalendarDatePicker", "CalendarDateTimePicker", "AutoCompleteField",
           "LinkRemoteFunction", "RemoteForm", "AjaxGrid", "URLLink"]

class CalendarDatePicker(FormField):
    css = [CSSLink(static, "calendar/calendar-system.css")]
    template = """
    <div xmlns:py="http://purl.org/kid/ns#">
    <input type="text" id="${field_id}" class="${field_class}" name="${name}" value="${strdate}" py:attrs="attrs"/>
    <input type="button" id="${field_id}_trigger" class="date_field_button" value="${button_text}" />
    <script type="text/javascript">
    Calendar.setup(
    {
        inputField : "${field_id}",
        ifFormat : "${format}",
        button : "${field_id}_trigger"
        <span py:if="picker_shows_time" py:replace="', showsTime : true'" />
    }
    );
    </script>
    </div>
    """
    params = ["button_text", "format", "picker_shows_time", "attrs"]
    attrs = {}
    picker_shows_time = False
    button_text = "Choose"
    format = "%m/%d/%Y"
    _default = None

    def __init__(self, name=None, default=None, not_empty=True,
                 calendar_lang=None, validator=None, **kw):
        """
        Use a javascript calendar system to allow picking of calendar dates.
        The format is in mm/dd/yyyy unless otherwise specified
        """
        super(CalendarDatePicker, self).__init__(name, **kw)
        self.not_empty = not_empty
        if default is not None or not self.not_empty:
            self._default = default
        self.validator = validator or validators.DateTimeConverter(
            format=self.format, not_empty=self.not_empty)

        lang_file_link = self.get_calendar_lang_file_link(calendar_lang)
        self.javascript=[JSLink(static, "calendar/calendar.js"),
                         JSLink(static, "calendar/calendar-setup.js"),
                         lang_file_link]

    def get_calendar_lang_file_link(self, calendar_lang):
        """
        Returns a CalendarLangFileLink containing a list of name
        patterns to try in turn to find the correct calendar locale
        file to use.
        """
        patterns = []
        
        # The preferred method is to use the explicitly specified
        # language.
        if calendar_lang is not None:
            patterns += ["calendar/lang/calendar-%s.js" % calendar_lang]

        # The next best method is to determine the language to use
        # from the HTTP header settings.
        patterns += ["calendar/lang/calendar-%(lang)s-%(charset)s.js",
                     "calendar/lang/calendar-%(lang)s.js"]

        # Fallback on English if neither of the above gave a result.
        patterns += ["calendar/lang/calendar-en.js"]
        
        return CalendarLangFileLink(static, patterns)

    def _get_default(self):
        if self._default is None and self.not_empty:
            return datetime.now()
        return self._default
    default = property(_get_default)

    def update_params(self, d):
        super(CalendarDatePicker, self).update_params(d)
        if hasattr(d['value'], 'strftime'):
            d['strdate'] = d['value'].strftime(d['format'])
        else:
            d['strdate'] = d['value']

class CalendarDatePickerDesc(CoreWD):
    name = "Calendar"
    for_widget = CalendarDatePicker("date_picker")


class CalendarDateTimePicker(CalendarDatePicker):
    format = "%Y/%m/%d %H:%M"
    picker_shows_time = True

class CalendarDateTimePickerDesc(CoreWD):
    name = "Calendar with time"
    for_widget = CalendarDateTimePicker("datetime_picker")

class AutoCompleteField(CompoundFormField):
    """Performs Ajax-style autocompletion by requesting search
    results from the server as the user types."""

    template = """
    <div xmlns:py="http://purl.org/kid/ns#">
    <script language="JavaScript" type="text/JavaScript">
        AutoCompleteManager${field_id} = new AutoCompleteManager('${field_id}',
        '${text_field.field_id}', '${hidden_field.field_id}',
        '${search_controller}', '${search_param}', '${result_name}',${str(only_suggest).lower()},
        '${tg.url([tg.widgets, 'turbogears.widgets/spinner.gif'])}', ${complete_delay}, ${str(take_focus).lower()});
        addLoadEvent(AutoCompleteManager${field_id}.initialize);
    </script>

    ${text_field.display(value_for(text_field), **params_for(text_field))}
    <img name="autoCompleteSpinner${name}" id="autoCompleteSpinner${field_id}" src="${tg.url([tg.widgets, 'turbogears.widgets/spinnerstopped.png'])}" alt="" />
    <div class="autoTextResults" id="autoCompleteResults${field_id}"/>
    ${hidden_field.display(value_for(hidden_field), **params_for(hidden_field))}
    </div>
    """
    javascript = [mochikit, JSLink(static,"autocompletefield.js")]
    css = [CSSLink(static,"autocompletefield.css")]
    member_widgets = ["text_field", "hidden_field"]
    params = ["search_controller", "search_param", "result_name",
              "attrs", "only_suggest", "complete_delay", "take_focus"]
    text_field = TextField(name="text")
    hidden_field = HiddenField(name="hidden")
    attrs = {}
    search_controller = ""
    search_param = "searchString"
    result_name = "textItems"
    only_suggest = False
    complete_delay = 0.200
    take_focus = False

class AutoCompleteFieldDesc(CoreWD):
    name = "Auto Complete"

    states = ["Alabama", "Alaska", "Arizona", "Arkansas", "California",
              "Colorado", "Connecticut", "Delaware", "Florida",
              "Georgia", "Hawaii", "Idaho", "Illinois", "Indiana",
              "Iowa", "Kansas", "Kentucky", "Louisiana", "Maine",
              "Maryland", "Massachusetts", "Michigan", "Minnesota",
              "Mississippi", "Missouri", "Montana", "Nebraska",
              "Nevada", "New Hampshire", "New Jersey", "New Mexico",
              "New York", "North Carolina", "North Dakota", "Ohio",
              "Oklahoma", "Oregon", "Pennsylvania", "Rhode Island",
              "South Carolina", "South Dakota", "Tennessee", "Texas",
              "Utah", "Vermont", "Virginia", "Washington",
              "West Virginia", "Wisconsin", "Wyoming"]

    template = """
    <div>
        Please choose a state:<br/>
        ${for_widget.display()}
    </div>
    """
    full_class_name = "turbogears.widgets.AutoCompleteField"

    def __init__(self, *args, **kw):
        super(AutoCompleteFieldDesc, self).__init__(*args, **kw)
        self.for_widget = AutoCompleteField(name="state",
                                    search_controller="%s/search" %
                                                    self.full_class_name,
                                    search_param="statename",
                                    result_name="states")

    def search(self, statename):
        statename = statename.lower()
        return dict(states=
                filter(lambda item: statename in item.lower(), self.states))
    search = expose(format="json")(search)

class LinkRemoteFunction(RPC):
    """ Returns a link that executes a POST asynchronously
    and updates a DOM Object with the result of it """

    template = """
    <a xmlns:py="http://purl.org/kid/ns#" name="${name}" py:attrs="attrs" onclick="${js}" href="#">${value}</a>
    """
    params = ["attrs"]
    attrs = {}

class LinkRemoteFunctionDesc(CoreWD):
    name = "Ajax remote function"

    states = ["Alabama", "Alaska", "Arizona", "Arkansas", "California",
              "Colorado", "Connecticut", "Delaware", "Florida",
              "Georgia", "Hawaii", "Idaho", "Illinois", "Indiana",
              "Iowa", "Kansas", "Kentucky", "Louisiana", "Maine",
              "Maryland", "Massachusetts", "Michigan", "Minnesota",
              "Mississippi", "Missouri", "Montana", "Nebraska",
              "Nevada", "New Hampshire", "New Jersey", "New Mexico",
              "New York", "North Carolina", "North Dakota", "Ohio",
              "Oklahoma", "Oregon", "Pennsylvania", "Rhode Island",
              "South Carolina", "South Dakota", "Tennessee", "Texas",
              "Utah", "Vermont", "Virginia", "Washington",
              "West Virginia", "Wisconsin", "Wyoming"]

    template = """
    <div>
        ${for_widget.display("States starting with 'N'", update="items")}
        <div id="items">states</div>
    </div>
    """
    full_class_name = "turbogears.widgets.LinkRemoteFunction"

    def __init__(self, *args, **kw):
        super(LinkRemoteFunctionDesc, self).__init__(*args, **kw)
        self.for_widget = LinkRemoteFunction(
            name="state", action="%s/search_linkrf" % self.full_class_name,
            data = dict(state_starts_with="N"))

    def search_linkrf(self, state_starts_with):
        return '<br />'.join(
                filter(lambda item: item.startswith(state_starts_with),
                       self.states)
       )
    search_linkrf = expose()(search_linkrf)

class RemoteForm(RPC, TableForm):
    """A TableForm that submits the data asynchronously and loads the resulting
    HTML into a DOM object"""

    def update_params(self, d):
        super(RemoteForm, self).update_params(d)
        d['form_attrs']['onSubmit'] = "return !remoteFormRequest(this, '%s', %s);" % (
            d.get("update", ''),
            jsonify.encode(self.get_options(d)),
        )


class RemoteFormDesc(CoreWD):
    name = "AJAX Form"

    class TestFormFields(WidgetsList):
        name = TextField()
        age = TextField()
        check = CheckBox()
        radio = RadioButtonList(options=[(1, "Python"), 
                                         (2, "Java"), 
                                         (3, "Pascal"),
                                         (4, "Ruby")],
                                default=4)

    template = """
    <div>
        ${for_widget.display()}
        <div id="post_data">&nbsp;</div>
    </div>
    """
    full_class_name = "turbogears.widgets.RemoteForm"

    def __init__(self, *args, **kw):
        super(RemoteFormDesc, self).__init__(*args, **kw)
        self.for_widget = RemoteForm(
            fields = self.TestFormFields(),
            name="remote_form",
            update = "post_data",
            action = "%s/post_data_rf" % self.full_class_name,
            before = "alert('pre-hook')",
            confirm = "Confirm?",
        )

    def post_data_rf(self, **kw):
        return """Received data:<br />%r""" % kw
    post_data_rf = expose()(post_data_rf)

ajaxgridcounter = itertools.count()

class AjaxGrid(Widget):
    """ AJAX updateable datagrid based on widget.js grid. """

    template = """<div id="${id}" xmlns:py="http://purl.org/kid/ns#">
    <a py:if="refresh_text"
       href="#"
       onclick="javascript:${id}_AjaxGrid.refresh(${defaults});return false;">
       ${refresh_text}
    </a>
    <div id="${id}_update"></div>
    <script type="text/javascript">
        addLoadEvent(partial(${id}_AjaxGrid.refresh, ${defaults}));
    </script>
    </div>
    """

    params = ["refresh_text", "id", "defaults"]

    defaults = {}
    refresh_text = "Update"
    id = "ajaxgrid_%d" % ajaxgridcounter.next()

    def __init__(self, refresh_url, *args, **kw):
        super(AjaxGrid, self).__init__(*args, **kw)
        target = "%s_update" % self.id
        self.javascript = [
            mochikit,
            JSLink("turbogears", "js/widget.js"),
            JSLink(static, "ajaxgrid.js"),
            JSSource("""
                %(id)s_AjaxGrid = new AjaxGrid('%(refresh_url)s', '%(target)s');
            """ % dict(id=self.id, refresh_url=refresh_url, target=target)
            ),
        ]

    def update_params(self, d):
        super(AjaxGrid, self).update_params(d)
        d["defaults"] = jsonify.encode(d["defaults"])

class AjaxGridDesc(CoreWD):
    name = "Ajax Grid"

    def facgen(n):
        total = 1
        yield 0, 1
        for x in xrange(1, n+1):
            total *= x
            yield x, total
    facgen = staticmethod(facgen)


    full_class_name = "turbogears.widgets.AjaxGrid"

    def __init__(self, *args, **kw):
        super(AjaxGridDesc, self).__init__(*args, **kw)
        self.for_widget = AjaxGrid(
            refresh_url = "%s/update" % self.full_class_name,
            # Dummy default params, just POC
            defaults = dict(),
        )
        self.update_count = itertools.count()

    def update(self):
        return dict(
            headers = ["N", "fact(N)"],
            rows = list(self.facgen(self.update_count.next())),
        )
    update = expose(format="json")(update)


class URLLink(FormField):
    template = """
    <a xmlns:py="http://purl.org/kid/ns#"
       href="$link"
       target="$target"
       py:attrs="attrs"
    >$text</a>
    """
    params = ["target", "text", "link", "attrs"]
    attrs = {}
    params_doc = {'link': 'Hyperlink',
                  'target': 'Specify where the link should be opened',
                  'text': 'The message to be shown for the link',
                  'attrs': 'Extra attributes'}



import turbogears
import cherrypy
from turbogears import widgets
from turbogears import controllers
from turbogears import validators
from turbogears import testutil

myform = widgets.TableForm(fields = [ 
    widgets.FieldSet(
        name = "p_data",
        fields = [
            widgets.TextField(name="name"),
            widgets.TextField(name="age", 
                validator=validators.Int()),
        ]),
])

class MyRoot(controllers.RootController):
    def testform(self, p_data, tg_errors=None):
        if tg_errors:
            self.has_errors = True
        self.name = p_data['name']
        self.age = p_data['age']
    testform = turbogears.validate(form=myform)(testform)
    testform = turbogears.expose(html="turbogears.tests.othertemplate")(
                                 testform)

    def set_errors(self):
        self.has_errors = True

    def testform_new_style(self, p_data):
        self.name = p_data['name']
        self.age = p_data['age']
    testform_new_style = turbogears.validate(form=myform)(testform_new_style)
    testform_new_style = turbogears.error_handler(set_errors)(testform_new_style)
    testform_new_style = turbogears.expose()(testform_new_style)


def test_form_translation_new_style():
    "Form input is translated into properly converted parameters"
    root = MyRoot()
    cherrypy.root = root
    testutil.createRequest("/testform_new_style?p_data.name=ed&p_data.age=5")
    assert root.name == "ed"
    print root.age
    assert root.age == 5

def test_invalid_form_with_error_handling():
    "Invalid forms can be handled by the method"
    root = cherrypy.root
    testutil.createRequest("/testform_new_style?p_data.name=ed&p_data.age=edalso")
    assert root.has_errors



import sys
from itertools import izip, repeat, islice
from inspect import getargspec

import cherrypy
from dispatch import generic, NoApplicableMethods, strategy

from turbogears.util import inject_args, adapt_call, call_on_stack, has_arg, \
                            remove_keys, Enum, combine_contexts
from turbogears.decorator import func_eq
from turbogears.genericfunctions import MultiorderGenericFunction


default = strategy.default

def dispatch_error(controller, tg_source, tg_errors, tg_exceptions,
                   *args, **kw):
    """Dispatch error.

    Error handler is a function registered via register_handler or if no
    such decorator was applied, the method triggering the error.
    """
dispatch_error = generic(MultiorderGenericFunction)(dispatch_error)

def _register_implicit_errh(controller, tg_source, tg_errors,
                                     tg_exceptions, *args, **kw):
    """Register implicitly declared error handler and re-dispatch.

    Any method declaring tg_errors parameter is considered an implicitly
    declared error handler.
    """
    error_handler(tg_source)(tg_source)
    return dispatch_error(controller, tg_source, tg_errors, tg_exceptions,
                          *args, **kw)
_register_implicit_errh = dispatch_error.when(
    "(tg_errors and has_arg(tg_source, 'tg_errors'))", order=3)(
    _register_implicit_errh)

def _register_implicit_exch(controller, tg_source, tg_errors,
                                         tg_exceptions, *args, **kw):
    """Register implicitly declared exception handler and re-dispatch.

    Any method declaring tg_exceptions parameter is considered an
    implicitly declared exception handler.
    """
    exception_handler(tg_source)(tg_source)
    return dispatch_error(controller, tg_source, tg_errors, tg_exceptions,
                          *args, **kw)
_register_implicit_exch = dispatch_error.when(
    "(tg_exceptions and has_arg(tg_source, 'tg_exceptions'))", order=3)(
    _register_implicit_exch)

def dispatch_error_adaptor(func):
    """Construct a signature isomorphic to dispatch_error.

    The actual handler will receive only arguments explicitly
    declared.
    """
    def adaptor(controller, tg_source, tg_errors, tg_exceptions, *args, **kw):
        args, kw = inject_args(func, {"tg_source":tg_source,
                                      "tg_errors":tg_errors,
                                      "tg_exceptions":tg_exceptions},
                               args, kw, 1)
        args, kw = adapt_call(func, args, kw, 1)
        return func(controller, *args, **kw)
    return adaptor

def try_call(func, self, *args, **kw):
    """Call function, catch and dispatch any resulting exception."""
    # turbogears.database import here to avoid circular imports
    from turbogears.database import _use_sa
    try:
        return func(self, *args, **kw)
    except Exception, e:
        if isinstance(e, cherrypy.HTTPRedirect) or \
           call_on_stack("dispatch_error",
                         {"tg_source":func, "tg_exception":e}, 4):
            raise
        elif _use_sa() and getattr(cherrypy.request, "in_transaction", None):
            # We're in a transaction and using SA, let 
            # database.run_with_transaction handle and dispatch 
            # the exception
            raise
        else:
            exc_type, exc_value, exc_trace = sys.exc_info()
            remove_keys(kw, ("tg_source", "tg_errors", "tg_exceptions"))
            try:
                output = dispatch_error(self, func, None, e, *args, **kw)
            except NoApplicableMethods:
                raise exc_type, exc_value, exc_trace
            else:
                del exc_trace
                return output

def run_with_errors(errors, func, self, *args, **kw):
    """Branch execution depending on presence of errors."""
    if errors:
        if hasattr(self, "validation_error"):
            import warnings
            warnings.warn(
                "Use decorator error_handler() on per-method base "
                "rather than defining a validation_error() method.",
                DeprecationWarning, 2)
            return self.validation_error(func.__name__, kw, errors)
        else:
            remove_keys(kw, ("tg_source", "tg_errors", "tg_exceptions"))
            try:
                return dispatch_error(self, func, errors, None, *args, **kw)
            except NoApplicableMethods:
                raise NotImplementedError("Method %s.%s() has no applicable "
                  "error handler." % (self.__class__.__name__, func.__name__))
    else:
        return func(self, *args, **kw)

def register_handler(handler=None, rules=None):
    """Register handler as an error handler for decorated method.

    If handler is not given, method is considered it's own error handler.

    rules can be a string containing an arbitrary logical Python expression
    to be used as dispatch rule allowing multiple error handlers for a
    single method.

    register_handler decorator is an invariant.
    """
    def register(func):
        when = "func_eq(tg_source, func)"
        if rules:
            when += " and (%s)" % rules
        dispatch_error.when(dispatch_error.parse(when, *combine_contexts(
            depth=[0, 1])), order=1)(dispatch_error_adaptor(handler or func))
        return func
    return register

def bind_rules(pre_rules):
    """Prepend rules to error handler specialisation."""
    def registrant(handler=None, rules=None):
        when = pre_rules
        if rules:
            when += " and (%s)" % rules
        return register_handler(handler, when)
    return registrant

error_handler = bind_rules("tg_errors")
exception_handler = bind_rules("tg_exceptions")

FailsafeSchema = Enum("none", "values", "map_errors", "defaults")

def dispatch_failsafe(schema, values, errors, source, kw):
    """Dispatch fail-safe mechanism for failed inputs."""
dispatch_failsafe = generic()(dispatch_failsafe)

def _failsafe_none(schema, values, errors, source, kw):
    """No fail-safe values."""
    return kw
_failsafe_none = dispatch_failsafe.when(strategy.default)(_failsafe_none)

def _failsafe_values_dict(schema, values, errors, source, kw):
    """Map errorneus inputs to values."""
    kw.update([(key, values[key]) for key in errors.iterkeys()
                                      if key in values])
    return kw
_failsafe_values_dict = dispatch_failsafe.when(
    "schema is FailsafeSchema.values and isinstance(values, dict) and "
    "isinstance(errors, dict)")(_failsafe_values_dict)

def _failsafe_values_atom(schema, values, errors, source, kw):
    """Map all errorneus inputs to a single value."""
    kw.update(izip(errors.iterkeys(), repeat(values)))
    return kw
_failsafe_values_atom = dispatch_failsafe.when(
    "schema is FailsafeSchema.values and isinstance(errors, dict)")(
    _failsafe_values_atom)

def _failsafe_map_errors(schema, values, errors, source, kw):
    """Map errorneus inputs to coresponding exceptions."""
    kw.update(errors)
    return kw
_failsafe_map_errors = dispatch_failsafe.when(
    "schema is FailsafeSchema.map_errors and isinstance(errors, dict)")(
    _failsafe_map_errors)

def _failsafe_defaults(schema, values, errors, source, kw):
    """Map errorneus inputs to method defaults."""
    argnames, defaultvals = getargspec(source)[::3]
    defaults = dict(izip(islice(argnames, len(argnames) - len(defaultvals),
                         None), defaultvals))
    kw.update([(key, defaults[key]) for key in errors.iterkeys()
                                        if key in defaults])
    return kw
_failsafe_defaults = dispatch_failsafe.when(
    "schema is FailsafeSchema.defaults and isinstance(errors, dict)")(
    _failsafe_defaults)

__all__ = ["dispatch_error", "dispatch_error_adaptor", "try_call",
           "run_with_errors", "default", "register_handler", "FailsafeSchema",
           "dispatch_failsafe", "error_handler", "exception_handler",
           ]



from unittest import TestCase
from turbogears import widgets
import turbogears
from turbogears import controllers
from turbogears import validators
from turbogears import testutil
import cherrypy
from datetime import datetime

class MyFormFields(widgets.WidgetsList):
    #XXX: Since allow_extra_fields should be removed from validators.Schema we
    #      Need a validator for every input-expecting widget
    name = widgets.TextField(validator=validators.String())
    age = widgets.TextField(validator=validators.Int(), default=0)
    date = widgets.CalendarDatePicker(validator=validators.DateConverter(
                                            if_empty=datetime.now()))

myform = widgets.TableForm(fields=MyFormFields())
                           
class MyRoot(controllers.RootController):

    def index(self):
        return dict(form=myform)
    index = turbogears.expose(html="turbogears.tests.form")(index)

    def usemochi(self):
        return dict(mochi=turbogears.mochikit, form=myform)
    usemochi = turbogears.expose(html="turbogears.tests.form")(usemochi)

    def testform(self, name, date, age, tg_errors=None):
        if tg_errors:
            self.has_errors = True
        self.name = name
        self.age = age
        self.date = date
    testform = turbogears.validate(form=myform)(testform)
    testform = turbogears.expose(html="turbogears.tests.othertemplate")(
                                 testform)

    def testform_new_style(self, name, date, age):
        if cherrypy.request.validation_errors:
            self.has_errors = True
        self.name = name
        self.age = age
        self.date = date
    testform_new_style = turbogears.validate(form=myform)(testform_new_style)
    testform_new_style = turbogears.expose()(testform_new_style)


def test_form_translation():
    "Form input is translated into properly converted parameters"
    root = MyRoot()
    cherrypy.root = root
    testutil.createRequest("/testform?name=ed&date=11/05/2005&age=5")
    assert root.name == "ed"
    print root.age
    assert root.age == 5

def test_form_translation_new_style():
    "Form input is translated into properly converted parameters"
    root = MyRoot()
    cherrypy.root = root
    testutil.createRequest("/testform_new_style?name=ed&date=11/05/2005&age=5&")
    assert root.name == "ed"
    print root.age
    assert root.age == 5

def test_invalid_form_with_error_handling():
    "Invalid forms can be handled by the method"
    root = cherrypy.root
    testutil.createRequest("/testform?name=ed&age=edalso&date=11/05/2005")
    assert root.has_errors

def test_css_should_appear():
    "CSS should appear when asked for"
    root = cherrypy.root
    testutil.createRequest("/")
    print cherrypy.response.body[0]
    assert "calendar-system.css" in cherrypy.response.body[0]

def test_javascript_should_appear():
    "JavaScript should appear when asked for"
    root = cherrypy.root
    testutil.createRequest("/")
    print cherrypy.response.body[0]
    assert "calendar.js" in cherrypy.response.body[0]

def test_include_mochikit():
    "JSLinks (and MochiKit especially) can be included easily"
    root = cherrypy.root
    testutil.createRequest("/usemochi")
    print cherrypy.response.body[0]
    assert "MochiKit.js" in cherrypy.response.body[0]

def test_mochikit_everywhere():
    "MochiKit can be included everywhere by setting tg.mochikit_all"
    root = cherrypy.root
    turbogears.config.update({"global":{"tg.mochikit_all" : True}})
    testutil.createRequest("/")
    turbogears.config.update({"global":{"tg.mochikit_all" : False}})
    print cherrypy.response.body[0]
    assert "MochiKit.js" in cherrypy.response.body[0]

def test_include_widgets():   
    "Any widget Can be included everywhere by  setting tg.include_widgets"
    root = cherrypy.root
    turbogears.config.update({"global":{"tg.include_widgets" : ["turbogears.mochikit"]}})
    testutil.createRequest("/")
    turbogears.config.update({"global":{"tg.include_widgets" : None}})
    print cherrypy.response.body[0]
    assert "MochiKit.js" in cherrypy.response.body[0]


class State(object):
    counter = 0
class AddingValidator(validators.FancyValidator):
    def _to_python(self, value, state=None):
        state.counter += 1
        return value
class AddingSchema(validators.Schema):
    a = AddingValidator()
    b = AddingValidator()
class AddingNestedSchema(AddingSchema):
    c = AddingSchema()

class TestValidationState(TestCase):

    class Controller(controllers.RootController):
        def validate(self, a, b, c):
            return 'counter: %d' % cherrypy.request.validation_state.counter
        validate = turbogears.expose()(validate)
        validate = turbogears.validate(validators=AddingNestedSchema(), 
                                       state_factory=State)(validate)

    def __init__(self, *args, **kw):
        super(TestValidationState, self).__init__(*args, **kw)
        cherrypy.root = self.Controller()

    def test_counter_is_incremented(self):
        # parameter values are irrelevant
        url = '/validate?a=1&b=2&c.a=3&c.b=4'
        testutil.create_request(url)
        body = cherrypy.response.body[0]
        print body
        msg = "Validation state is not handled properly"
        # 4 == 1 (a) + 1(b) + 1(c.a) + 1(c.b)
        self.failUnless('counter: 4' in body, msg)

   



from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages
from pkg_resources import DistributionNotFound

import sys, os

if sys.version_info < (2, 4):
    raise SystemExit("Python 2.4 or later is required")

execfile(os.path.join("apigen", "release.py"))

setup(
    name="apigen",
    version=version,
    author=author,
    author_email=email,
    download_url="http://www.turbogears.org/download/",
    license=license,
    description="Packaged up PythonDoc for easy API generation",
    long_description="""
Easily generate decent looking API docs.""",
    url="http://www.turbogears.org",
    zip_safe=False,
    install_requires = [],
    packages=find_packages(),
    include_package_data=True,
    exclude_package_data={},
    entry_points = """
    [console_scripts]
    apigen = apigen.command:main
    """,
    extras_require = {
    },
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    test_suite = 'nose.collector',
    )
    



# Formatting configuration for locale sw_KE

languages={'sw': 'Kiswahili'}

countries={'DO': 'Jamhuri ya Dominikan', 'FR': 'Ufaransa', 'DM': 'Dominika', 'DJ': 'Jibuti', 'BA': 'Bosnia na Herzegowina', 'BE': 'Ubelgiji', 'HR': 'Kroatia', 'DE': 'Udachi', 'JP': 'Ujapani', 'VC': 'Saint Vincent na Grenadines', 'HU': 'Hungaria', 'JM': 'Jamaika', 'ZA': 'Afrika ya Kusini', 'MH': 'Visiwa vya Marshall', 'US': 'Muungano wa Nchi za Amerika', 'BR': 'Brazili', 'BS': 'Visiwa vya Bahama', 'CO': 'Kolombia', 'UY': 'Urugwai', 'DK': 'Udenmarki', 'IS': 'Barafu', 'NI': 'Nikaragua', 'NL': 'Uholanzi', 'CM': 'Kamerun', 'PT': 'Ureno', 'NO': 'Unorwe', 'TT': 'Trinidad na Tobago', 'PY': 'Paragwai', 'TR': 'Uturuki', 'TL': 'Timor ya Mashariki', 'LU': 'Luksemburg', 'PG': 'Papua Guinea Mpya', 'NP': 'Nepali', 'PH': 'Filipino', 'GQ': 'Guinea ya Ikweta', 'EG': 'Misri', 'VA': 'Vatikano', 'CI': 'Pwani ya Pembe', 'CH': 'Uswisi', 'AE': 'Muugano wa Falme za Nchi za Kiarabu', 'CN': 'Uchina', 'AG': 'Antigua na Barbuda', 'RU': 'Urusi', 'IQ': 'Iraki', 'YE': 'Yemeni', 'CA': 'Kanada', 'IR': 'Uajemi', 'CG': 'Kongo', 'CF': 'Jamhuri ya Afrika ya Kati', 'EC': 'Ekvado', 'IT': 'Uitaliani', 'CZ': 'Jamhuri ya Czech', 'AR': 'Ajentina', 'GB': 'Uingereza', 'IN': 'Uhindi', 'ET': 'Uhabeshi', 'CV': 'Rasi Verde', 'ES': 'Uhispania', 'MA': 'Moroko', 'MC': 'Monako', 'KE': 'Kenya', 'KH': 'Kampuchea', 'KN': 'Saint Kitts na Nevis', 'KM': 'Visiwa vya Komoro', 'ST': 'Sao Tome na Principe', 'KR': 'Korea ya Kusini', 'TD': 'Chadi', 'KP': 'Korea ya Kaskazini', 'SO': 'Somali', 'TZ': 'Tanzania', 'SC': 'Visiwa vya Shelisheli', 'SB': 'Visiwa vya Solomon', 'SA': 'Arabuni Saudi', 'MZ': 'Msumbiji', 'MY': 'Malasya', 'MX': 'Meksiko', 'SE': 'Uswidi', 'IL': 'Uyahudi'}

months=['Januari', 'Februari', 'Machi', 'Aprili', 'Mei', 'Juni', 'Julai', 'Agosti', 'Septemba', 'Oktoba', 'Novemba', 'Desemba']

abbrMonths=['Jan', 'Feb', 'Mar', 'Apr', 'Mei', 'Jun', 'Jul', 'Ago', 'Sep', 'Okt', 'Nov', 'Des']

days=['Jumatatu', 'Jumanne', 'Jumatano', 'Alhamisi', 'Ijumaa', 'Jumamosi', 'Jumapili']

abbrDays=['Jtt', 'Jnn', 'Jtn', 'Alh', 'Iju', 'Jmo', 'Jpi']

dateFormats={'medium': '%Y %%(abbrmonthname)s %d ', 'full': '%%(dayname)s, %Y %%(monthname)s %d ', 'long': '%Y %%(monthname)s %d ', 'short': '%y/%m/%d'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}


# Formatting configuration for locale so_SO

languages={'so': 'Soomaali'}

countries={'BD': 'Bangaala-Deesh', 'BE': 'Beljiyam', 'BA': 'Boosniya Heersigoviina', 'BB': 'Baarbadoos', 'BH': 'Baxrayn', 'BJ': 'Beniin', 'JM': 'Jameyka', 'JO': 'Urdun', 'BR': 'Braasiil', 'RU': 'Ruush', 'RO': 'Rumaaniya', 'GR': 'Giriigga', 'JP': 'Jabbaan', 'GD': 'Giriinaada', 'GN': 'Gini', 'OM': 'Cumaan', 'TN': 'Tuniisiya', 'IL': "Israa'iil", 'HR': 'Korweeshiya', 'HU': 'Hangeri', 'PT': 'Bortuqaal', 'PK': 'Bakistaan', 'PH': 'Filibiin', 'PL': 'Booland', 'ZM': 'Saambiya', 'EG': 'Masar', 'ZA': 'Koonfur Afrika', 'VN': 'Fiyetnaam', 'KZ': 'Kasaakhistaan', 'ET': 'Itoobiya', 'ZW': 'Simbaabwe', 'ES': 'Isbeyn', 'MA': 'Marooko', 'MC': 'Moonako', 'ML': 'Maali', 'MK': 'Makadooniya', 'MT': 'Maalda', 'MW': 'Malaawi', 'MV': 'Maaldiqeen', 'MR': 'Muritaaniya', 'UG': 'Ugaanda', 'MX': 'Meksiko', 'MZ': 'Musambiig', 'FR': 'Faransiis', 'FI': 'Fiinlaand', 'NI': 'Nikaraaguwa', 'NO': 'Noorweey', 'NA': 'Namiibiya', 'NG': 'Nayjeeriya', 'NZ': 'Neyuusilaand', 'CH': 'Swiiserlaand', 'CN': 'Shiinaha', 'CM': 'Kameruun', 'CL': 'Jili', 'CA': 'Kanada', 'CU': 'Kuuba', 'SY': 'Suuriya', 'KE': 'Kiiniya', 'KH': 'Kamboodiya', 'KR': 'Kuuriyada Koonfureed', 'KP': 'Kuuriyada Waqooyi', 'SO': 'Soomaaliya', 'SL': 'Siraaliyoon', 'KW': 'Kuwayt', 'SA': 'Sacuudi Carabiya', 'SE': 'Iswidhan', 'SD': 'Sudaan', 'DJ': 'Jabuuti', 'DK': 'Danmaark', 'DE': 'Jarmal', 'YE': 'Yaman', 'US': 'Qaramada Midoobey ee Maraykanka', 'LB': 'Lubnaan', 'TR': 'Turki', 'LK': 'Siirilaanka', 'LV': 'Laatfiya', 'LU': 'Luksemboorg', 'LR': 'Laybeeriya', 'LS': 'Losooto', 'TH': 'Taylaand', 'TG': 'Toogo', 'TD': 'Jaad', 'LY': 'Liibiya', 'VA': 'Faatikaan', 'AE': 'Imaaraadka Carabta ee Midoobay', 'VE': 'Fenisuweela', 'AF': 'Afgaanistaan', 'IQ': 'Ciraaq', 'IS': 'Iislaand', 'IR': 'Iiraan', 'AM': 'Armeeniya', 'IT': 'Talyaani', 'AO': 'Angoola', 'AU': 'Awstraaliya', 'AT': 'Osteeriya', 'IN': 'Hindiya', 'TZ': 'Tansaaniya', 'IE': 'Ayrlaanda', 'ID': 'Indoneesiya', 'QA': 'Qadar'}

months=['Bisha Koobaad', 'Bisha Labaad', 'Bisha Saddexaad', 'Bisha Afraad', 'Bisha Shanaad', 'Bisha Lixaad', 'Bisha Todobaad', 'Bisha Sideedaad', 'Bisha Sagaalaad', 'Bisha Tobnaad', 'Bisha Kow iyo Tobnaad', 'Bisha Laba iyo Tobnaad']

abbrMonths=['Kob', 'Lab', 'Sad', 'Afr', 'Sha', 'Lix', 'Tod', 'Sid', 'Sag', 'Tob', 'KIT', 'LIT']

days=['Isniin', 'Salaaso', 'Arbaco', 'Khamiis', 'Jimco', 'Sabti', 'Axad']

abbrDays=['Isn', 'Sal', 'Arb', 'Kha', 'Jim', 'Sab', 'Axa']

dateFormats={'medium': '%d-%%(abbrmonthname)s-%Y', 'full': '%%(dayname)s, %%(monthname)s %d, %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



from turbogears.widgets.datagrid import DataGrid

class Foo:
    def __init__(self, name, subtype, text):
        self.name = name
        self.subtype = subtype
        self.text = text
    def _get_excerpt(self):
        return self.text + '...'
    excerpt = property(fget=_get_excerpt)

class User:
    def __init__(self, ID, name, emailAddress):
        self.userId = ID
        self.name = name
        self.emailAddress = emailAddress
    displayName = property(fget=lambda self: self.name.capitalize())

class TestDataGrid:
    def test_declaration_styles(self):
        grid = DataGrid(name='grid', fields=[
                    DataGrid.Column('name', options=dict(foobar=123)), 
                    ('Subtype', 'subtype'),
                    DataGrid.Column('text', 'excerpt', 'TEXT'),
                ])
        d = dict(value='value')
        grid.update_params(d)
        get_field = d['get_field']
        assert ['Name', 'Subtype', 'TEXT'] == d['headers']
        assert ['name', 'column-1', 'text'] == d['collist']
        assert d['columns'][0].get_option('foobar') == 123
        assert grid.get_column('name').options['foobar'] == 123
        row = Foo('spa1', 'fact', 'thetext')
        assert 'spa1' == get_field(row, 'name')
        assert 'fact' == get_field(row, 'column-1')
        assert 'thetext...' == get_field(row, 'text')
    def test_template_overridal(self):
        grid = DataGrid(fields=[
                    ('Name', 'name'), 
                    ('Subtype', 'subtype'),
                    ('TEXT', Foo._get_excerpt),
                ], template = "turbogears.fastdata.templates.datagrid")
        d = dict(value='value')
        grid.update_params(d)
    def test_wiki_samples(self):
        #Test that sample code posted on DataGridWidget wiki page actually works.
        grid = DataGrid(fields=[
            ('ID', 'userId'),
            ('Name', 'displayName'),
            ('E-mail', 'emailAddress'),
        ])
        users = [User(1, 'john', 'john@foo.net'), User(2, 'fred', 'fred@foo.net')]
        print grid.display(users)
        grid = DataGrid(fields=[
            ('Name', lambda row: row[1]),
            ('Country', lambda row: row[2]),
            ('Age', lambda row: row[0]),
        ])
        data = [(33, "Anton Bykov", "Bulgaria"), 
            (23, "Joe Doe", "Great Britain"), (44, "Pablo Martelli", "Brazil")]
        print grid.display(data)






import os.path
import tempfile

from turbogears.i18n import sogettext
import turbogears

def setup_module():
    basedir = os.path.join(os.path.dirname(os.path.dirname(
            os.path.dirname(os.path.abspath(__file__)))), "tests")
    locale_dir = os.path.join(basedir, 'locale')
    turbogears.config.update({
        'i18n.locale_dir':locale_dir,
        'i18n.domain':'messages',
        'i18n.default_locale':'en',
        'i18n.get_locale':lambda: 'en',
        'i18n.run_template_filter':False,
        'sqlobject.dburi':"sqlite:///" + tempfile.gettempdir()  + "/testlocaledb"
    })
    sogettext.create_so_catalog(["en","fi"], "messages")




import itertools
from copy import copy
from inspect import getargspec, formatargspec

from peak.util.decorators import decorate_assignment


# Inspired by Michele Simionato's decorator library
# http://www.phyast.pitt.edu/~micheles/python/documentation.html

def decorate(func, caller, signature=None):
    """Decorate func with caller."""
    if signature is not None:
        argnames, varargs, kwargs, defaults = signature
    else:
        argnames, varargs, kwargs, defaults = getargspec(func)
    if defaults is None:
        defaults = ()
    parameters = formatargspec(argnames, varargs, kwargs, defaults)[1:-1]
    defval = itertools.count(len(argnames)-len(defaults))
    args = formatargspec(argnames, varargs, kwargs, defaults,
                         formatvalue=lambda value:"=%s" % (
                         argnames[defval.next()]))[1:-1]

    func_str = """
def %s(%s):
  return caller(func, %s)
""" % (func.__name__, parameters, args)

    exec_dict = dict(func=func, caller=caller)
    exec func_str in exec_dict
    newfunc = exec_dict[func.__name__]
    newfunc.__doc__ = func.__doc__
    newfunc.__dict__ = func.__dict__.copy()
    newfunc.__module__ = func.__module__
    if hasattr(func, "__composition__"):
        newfunc.__composition__ = copy(func.__composition__)
    else:
        newfunc.__composition__ = [func]
    newfunc.__composition__.append(newfunc)
    return newfunc

def decorator(entangler, signature=None):
    """Decorate function with entangler.

    Use signature as signature or preserve original signature if signature
    is None.

    Enables alternative decorator syntax for Python 2.3 as seen in PEAK:

        [my_decorator(foo)]
        def baz():
            pass

    Mind, the decorator needs to be a closure for this syntax to work.
    """
    def callback(frame, k, v, old_locals):
        return decorate(v, entangler(v), signature)
    return decorate_assignment(callback, 3)

def weak_signature_decorator(entangler):
    """Decorate function with entangler and change signature to accept
    arbitrary additional arguments.

    Enables alternative decorator syntax for Python 2.3 as seen in PEAK:

        [my_decorator(foo)]
        def baz():
            pass

    Mind, the decorator needs to be a closure for this syntax to work.
    """
    def callback(frame, k, v, old_locals):
        return decorate(v, entangler(v), make_weak_signature(v))
    return decorate_assignment(callback, 3)

def simple_decorator(caller, signature=None):
    """Decorate function with caller."""
    def entangle(func):
        return decorate(func, caller, signature)
    return entangle

def simple_weak_signature_decorator(caller):
    """Decorate function with caller and change signature to accept
    arbitrary additional arguments."""
    def entangle(func):
        return decorate(func, caller, make_weak_signature(func))
    return entangle

def make_weak_signature(func):
    """Change signature to accept arbitrary additional arguments."""
    argnames, varargs, kwargs, defaults = getargspec(func)
    if kwargs is None:
        kwargs = "_decorator__kwargs"
    if varargs is None:
        varargs = "_decorator__varargs"
    return argnames, varargs, kwargs, defaults

def compose(*decorators):
    """Compose decorators."""
    return lambda func: reduce(lambda f, g: g(f), decorators, func)

def func_composition(func):
    """Return composition (decorator wise) of function."""
    return getattr(func, "__composition__", [func])

def func_original(func):
    """Return original (undecorated) function."""
    return func_composition(func)[0]

def func_id(func):
    """Return identity of function.

    Identity is invariant under decorator application (if decorator is
    created with decorator() or weak_signature_decorator()).
    """
    return id(func_original(func))

def func_eq(f, g):
    """Check if functions are identical."""
    return func_id(f) == func_id(g)

__all__ = ["decorator", "compose", "func_id", "func_eq", "func_original",
           "func_composition", "weak_signature_decorator", "decorate",
           "make_weak_signature", "simple_decorator",
           "simple_weak_signature_decorator",]



import unittest

from turbogears.decorator import *
from turbogears import testutil


def d1(func):
    def call(func, *args, **kw):
        return func(*args, **kw)
    return call
d1 = decorator(d1)

def d2(func):
    def call(func, *args, **kw):
        return func(*args, **kw)
    return call
d2 = decorator(d2)

def d3(func):
    func.baz2 = 2
    def call(func, *args, **kw):
        return func(*args, **kw)
    return call
d3 = decorator(d3)

def foo(a, b):
    return a+b
foo.baz1 = 1

bar = foo

foo = d1(foo)
foo = d2(foo)
foo = d3(foo)

def addn(n):
    def entangle(func):
        def call(func, *args, **kw):
            return func(*args, **kw) + n
        return call
    return decorator(entangle)

[addn(11)]
def py23(a):
    return a

[compose(addn(1), addn(2))]
def composed(a):
    return a

def weakener():
    def entangle(func):
        def call(func, *args, **kw):
            return len(args)
        return call
    return weak_signature_decorator(entangle)

[weakener()]
def weakling():
    pass

def sig_changer():
    def entangle(func):
        def call(func, a, b):
            return func(b)
        return call
    return decorator(entangle, (["a", "b"], None, None, None))

[sig_changer()]
def new_sig(a):
    return a

def simple(func, *args, **kw):
    return func(*args, **kw) + 1
simple = simple_decorator(simple)

def simple_weakener(func, *args, **kw):
     return len(args)
simple_weakener = simple_weak_signature_decorator(simple_weakener)

def simple_weakling():
    pass
simple_weakling = simple_weakener(simple_weakling)

def simple_adder1(a):
    return a
simple_adder1 = simple(simple_adder1)

class TestDecorator(unittest.TestCase):

    def test_preservation(self):
        for key, value in bar.__dict__.iteritems():
            self.failUnless(key in foo.__dict__)
            self.failUnless(value == foo.__dict__[key])
        self.failUnless(bar.__name__ == foo.__name__)
        self.failUnless(bar.__module__ == foo.__module__)
        self.failUnless(bar.__doc__ == foo.__doc__)

    def test_eq(self):
        self.failUnless(func_id(foo) == func_id(bar))
        self.failUnless(func_eq(foo, bar))
        self.failUnless(bar == func_original(foo))
        self.failUnless(bar is func_original(foo))

    def test_history(self):
        self.failUnless(len(func_composition(foo)) == 4)
        self.failUnless(func_composition(foo)[:-1] ==
                        func_composition(func_composition(foo)[-2]))

    def test_23compatibility(self):
        self.failUnless(py23(2)==13)

    def test_attributes(self):
        self.failUnless(foo.baz1==1)
        self.failUnless(foo.baz2==2)

    def test_composition(self):
        self.failUnless(addn(3)(py23)(2)==16)
        self.failUnless(composed(1)==4, composed(1))
        self.failUnless(composed.__name__ == "composed")

    def test_signature(self):
        self.failUnless(make_weak_signature(bar)[1:3]==(
                        "_decorator__varargs", "_decorator__kwargs"))
        self.failUnless(weakling(1,2)==2)
        self.failUnless(new_sig(1,2)==2)

    def test_simple_decorators(self):
        self.failUnless(simple_adder1(1)==2)
        self.failUnless(simple_weakling(1,2)==2)



import re
import types
from math import ceil
import logging

import cherrypy
try:
    import sqlobject
    from sqlobject.main import SelectResults
except ImportError:
    SelectResults = None
    sqlobject = None

try:
    # Can't depend on sqlalchemy being available.
    import sqlalchemy
    from sqlalchemy.ext.selectresults import SelectResults as SASelectResults
except ImportError:
    SASelectResults = None
    sqlalchemy = None
 
import turbogears
from turbogears.decorator import weak_signature_decorator
from turbogears.view import variable_providers
from formencode.variabledecode import variable_encode, variable_decode

log = logging.getLogger("turbogears.paginate")

def paginate(var_name, default_order='', default_reversed=False, limit=10,
            allow_limit_override=False, max_pages=5):
    def entangle(func):
        def decorated(func, *args, **kw):
            page = int(kw.pop('tg_paginate_no', 1))
            limit_ = int(kw.pop('tg_paginate_limit', limit))
            order = kw.pop('tg_paginate_order', None)
            ordering = kw.pop('tg_paginate_ordering', {})

            # Convert ordering str to a dict.
            if ordering:
                ordering = convert_ordering(ordering)

            if not allow_limit_override:
                limit_ = limit

            log.debug("Pagination params: page=%s, limit=%s, order=%s "
                      "", page, limit_, order)
            
            # get the output from the decorated function    
            output = func(*args, **kw)
            if not isinstance(output, dict):
                return output
            try:
                var_data = output[var_name]
            except KeyError:
                raise "Didn't get expected variable"
            
            if order and not default_order:
                raise "If you want to enable ordering you need " \
                      "to provide a default_order" 
            elif default_order and not ordering:
                ordering = {default_order:[0, not default_reversed]}
            elif ordering and order:
                sort_ordering(ordering, order)
            log.debug('ordering %s' % ordering)

            row_count = 0
            if (SelectResults and isinstance(var_data, SelectResults)) or \
               (SASelectResults and isinstance(var_data, SASelectResults)):
                row_count = var_data.count()
                if ordering:
                    # Build order_by list.
                    order_cols = range(len(ordering))
                    for (colname, order_opts) in ordering.items():
                        col = sql_get_column(colname, var_data)
                        if not col:
                            raise StandardError, "The order column (%s) doesn't exist" % colname
                        order_by_expr = sql_order_col(col, order_opts[1])
                        order_cols[order_opts[0]] = order_by_expr
                    # May need to address potential of ordering already
                    # existing in var_data.
                    # SO and SA differ on this method name.
                    if hasattr(var_data, 'orderBy'):
                        var_data = var_data.orderBy(order_cols)
                    else:
                        var_data = var_data.order_by(order_cols)
            elif isinstance(var_data, list):
                row_count = len(var_data)
            else:
                raise 'Variable is not a list or SelectResults'

            offset = (page-1) * limit_
            page_count = int(ceil(float(row_count)/limit_))

            # if it's possible display every page
            if page_count <= max_pages:
                pages_to_show = range(1,page_count+1)
            else:
                pages_to_show = _select_pages_to_show(page_count=page_count,
                                              current_page=page,
                                              max_pages=max_pages)
                
            # which one should we use? cherrypy.request.input_values or kw?
            #input_values = cherrypy.request.input_values.copy()
            ##input_values = kw.copy()
            input_values =  variable_encode(cherrypy.request.params.copy())
            input_values.pop('self', None)
            for input_key in input_values.keys():
                if input_key.startswith('tg_paginate'):
                    del input_values[input_key]

            cherrypy.request.paginate = Paginate(current_page=page,
                                             limit=limit_, 
                                             pages=pages_to_show, 
                                             page_count=page_count, 
                                             input_values=input_values, 
                                             order=order,
                                             ordering=ordering,
                                             row_count=row_count)
                                             
            # we replace the var with the sliced one
            endpoint = offset + limit_
            log.debug("slicing data between %d and %d", offset, endpoint)
            output[var_name] = var_data[offset:endpoint]

            return output
        return decorated
    return weak_signature_decorator(entangle)

def _paginate_var_provider(d): 
    # replaced cherrypy.thread_data for cherrypy.request
    # thanks alberto!
    paginate = getattr(cherrypy.request, 'paginate', None)
    if paginate:
        d.update(dict(paginate=paginate))
variable_providers.append(_paginate_var_provider)

class Paginate:
    """class for variable provider"""
    def __init__(self, current_page, pages, page_count, input_values, 
                 limit, order, ordering, row_count):
                 
        self.pages = pages
        self.limit = limit
        self.page_count = page_count
        self.current_page = current_page
        self.input_values = input_values
        self.order = order
        self.ordering = ordering
        self.row_count = row_count
        self.first_item = (current_page - 1) * limit + 1
        self.last_item = min(current_page * limit, row_count)
        self.reversed = False

        # Should reversed be true?
        for (field_name, ordering_values) in ordering.items():
            if ordering_values[0] == 0 and not ordering_values[1]:
                self.reversed = True

        # If ordering is empty, don't add it.
        input_values = dict(tg_paginate_limit=limit)
        if ordering:
            input_values['tg_paginate_ordering'] = ordering
        self.input_values.update(input_values)

        if current_page < page_count:
            self.input_values.update(dict(
                                tg_paginate_no=current_page+1,
                                tg_paginate_limit=limit))
            self.href_next = turbogears.url(cherrypy.request.path, input_values)
            self.input_values.update(dict(
                                tg_paginate_no=page_count,
                                tg_paginate_limit=limit))
            self.href_last = turbogears.url(cherrypy.request.path, input_values)
        else:
            self.href_next = None
            self.href_last = None
            
        if current_page > 1:
            self.input_values.update(dict(
                                tg_paginate_no=current_page-1,
                                tg_paginate_limit=limit))
            self.href_prev = turbogears.url(cherrypy.request.path, input_values)
            self.input_values.update(dict(
                                tg_paginate_no=1,
                                tg_paginate_limit=limit))
            self.href_first = turbogears.url(cherrypy.request.path, input_values)
        else:
            self.href_prev = None
            self.href_first = None
            
    def get_href(self, page, order=None, reverse_order=None):
        # Note that reverse_order is not used.  It should be cleaned up here
        # and in the template.  I'm not removing it now because I don't want
        # to break the API.
        order = order or None
        self.input_values['tg_paginate_no'] = page
        if order:
            self.input_values['tg_paginate_order'] = order

        return turbogears.url('', self.input_values)

def _select_pages_to_show(current_page, page_count, max_pages):
    pages_to_show = []
    
    if max_pages < 3:
        raise "The minimun value for max_pages on this algorithm is 3"

    if page_count <= max_pages:
        pages_to_show = range(1,page_count+1)
    
    pad = 0
    if not max_pages % 2:
        pad = 1
        
    start = current_page - (max_pages / 2) + pad
    end = current_page + (max_pages / 2)
    
    if start < 1:
        end = end + (start * -1) + 1
        start = 1

    if end > page_count:
        start = start - (end - page_count)
        end = page_count
        
    return range(start, end+1)

def sort_ordering(ordering, sort_name):
    """Rearrange ordering based on sort_name."""
    log.debug('sort called with %s and %s' % (ordering, sort_name))
    if sort_name not in ordering:
        ordering[sort_name] = [-1, True] 
    if ordering[sort_name][0] == 0:
        # Flip
        ordering[sort_name][1] = not ordering[sort_name][1]
    else:
        ordering[sort_name][0] = 0
        for key in ordering.keys():
            if key != sort_name and ordering[key][0] < len(ordering) - 1:
                ordering[key][0] += 1
    log.debug('sort results is %s and %s' % (ordering, sort_name))

def sql_get_column(colname, var_data):
    """Return a column from var_data based on colname."""
    if isinstance(var_data, SelectResults):
        col = getattr(var_data.sourceClass.q, colname, None)

    elif isinstance(var_data, SASelectResults):
        col = getattr(
                var_data._query.mapper.c,
                colname[len(var_data._query.mapper.column_prefix or ''):],
                None)

    else:
        raise StandardError, 'expected SelectResults'

    return col

def sql_order_col(col, ascending=True):
    """Return an ordered col for col."""
    if sqlalchemy and isinstance(col, sqlalchemy.schema.Column):
        if ascending:
            order_col = sqlalchemy.sql.asc(col)
        else:
            order_col = sqlalchemy.sql.desc(col)
    elif sqlobject and isinstance(col, types.InstanceType):
        # I don't like using InstanceType, but that's what sqlobject col type
        # is.
        if ascending:
            order_col = col
        else:
            order_col = sqlobject.DESC(col)
    else:
        raise StandardError, 'expected Column, but got %s' % str(type(col))
    return order_col
   
# Ordering re:
ordering_expr = re.compile(r"('\w+'): ?\[(\d+), ?(True|False)\]")

def convert_ordering(ordering):
    """Covert ordering unicode string to dict."""

    log.debug('ordering received %s' % str(ordering))

    # eval would be simple, but insecure.
    if not isinstance(ordering, (str, unicode)):
        raise ValueError, "ordering should be string or unicode."
    new_ordering = {}
    if ordering == u"{}":
        pass
    else:
        try:
            ordering_info_find = ordering_expr.findall(ordering)
            emsg = "Didn't match ordering for %s." % str(ordering)
            assert len(ordering_info_find) > 0, emsg
            for ordering_info in ordering_info_find:
                ordering_key = str(ordering_info[0]).strip("'")
                ordering_order = int(ordering_info[1])
                ordering_reverse = bool(ordering_info[2] == 'True')
                new_ordering[ordering_key] = [ordering_order,
                                              ordering_reverse]
        except StandardError, e:
            log.debug('FAILED to convert ordering.')
            new_ordering = {}
    log.debug('ordering converted to %s' % str(new_ordering))
    return new_ordering




"""
Generic widget to present and manipulate data in a grid (tabular) form.

See also: http://trac.turbogears.org/turbogears/wiki/DataGridWidget
"""

from turbogears.widgets import Widget, CSSLink, static
from turbogears.widgets.base import CoreWD

NoDefault = object()

__all__ = ["DataGrid", "PaginateDataGrid"]

class DataGrid(Widget):

    """Generic widget to present and manipulate data in a grid (tabular) form.

    The columns to build the grid from are specified with fields ctor argument
    which is a list.  Currently an element can be either a two-element tuple or
    instance of DataGrid.Column class. If tuple is used it a Column is then
    build out of it, first element is assumed to be a title and second element -
    field accessor.

    You can specify columns' data statically, via fields ctor parameter, or
    dynamically, by via 'fields' key.
    """

    css=[CSSLink(static, "grid.css")]
    template = "turbogears.widgets.templates.datagrid"
    fields = None

    class Column:
        """Simple struct that describes single DataGrid column.

        Column has:
          - a name, which allows to uniquely identify column in a DataGrid
          - getter, which is used to extract field's value
          - title, which is displayed in the table's header
          - options, which is a way to carry arbitrary user-defined data

        """
        def __init__(self, name, getter=None, title=None, options=None):
            if not name:
                raise ValueError, 'name is required'
            if getter:
                if callable(getter):
                    self.getter = getter
                else: # assume it's an attribute name
                    self.getter = DataGrid.attrwrapper(getter)
            else:
                self.getter = DataGrid.attrwrapper(name)
            self.name = name
            self.title = title or name.capitalize()
            self.options = options or {}
        def get_option(self, name, default=NoDefault):
            if name in self.options:
                return self.options[name]
            if default is NoDefault: # no such key and no default is given
                raise KeyError(name)
            return default
        def get_field(self, row):
            return self.getter(row)
        def __str__(self):
            return "<DataGrid.Column %s>" % self.name

    class attrwrapper:
        """Helper class that returns an object's attribute when called.

        This allows to access 'dynamic' attributes (properties) as well as
        simple static ones.
        """
        def __init__(self, name):
            assert isinstance(name, str)
            self.name = name
        def __call__(self, obj):
            return getattr(obj, self.name)

    def __init__(self, fields=None, **kw):
        super(DataGrid, self).__init__(**kw)
        if fields:
            self.fields = fields
        if self.fields is None:
            self.fields = []
        self.columns = self._parse(self.fields)
    def get_column(self, name):
        """Returns DataGrid.Column with specified name.
        Raises KeyError if no such column exists.
        """
        for col in self.columns:
            if col.name == name:
                return col
        raise KeyError(name)
    def __getitem__(self, name):
        """Shortcut to get_column."""
        return self.get_column(name)
    def get_field_getter(columns):
        """ Returns a function to access the fields of table by row, col """
        idx = {} # index columns by name
        for col in columns:
            idx[col.name] = col
        def _get_field(row, col):
            return idx[col].get_field(row)
        return _get_field
    get_field_getter = staticmethod(get_field_getter)
    def update_params(self, d):
        super(DataGrid, self).update_params(d)
        if d.get('fields'):
            fields = d.pop('fields')
            columns = self._parse(fields)
        else:
            columns = self.columns[:]
        d['columns'] = columns
        d['get_field'] = self.get_field_getter(columns)
        # this is for backward compatibility
        d['headers'] = [col.title for col in columns]
        d['collist'] = [col.name for col in columns]
    def _parse(self, fields):
        "Parses fields specification into a list of L{Columns}s."
        columns = []
        names = {} # keep track of names to ensure there are no dups
        for n,col in enumerate(fields):
            if not isinstance(col, self.Column):
                title, name_or_f = col
                # construct name using column index
                name = 'column-' + str(n)
                col = self.Column(name, name_or_f, title)
            if col.name in names:
                raise ValueError('duplicate column name: %s' % name)
            columns.append(col)
            names[col.name] = 1
        return columns

class DataGridDesc(CoreWD):
    name = "DataGrid"
    for_widget = DataGrid(fields=[('Name', lambda row: row[1]),
                                  ('Country', lambda row: row[2]),
                                  ('Age', lambda row: row[0])],
                          default=[(33, "Anton Bykov", "Bulgaria"),
                                   (23, "Joe Doe", "Great Britain"),
                                   (44, "Pablo Martelli", "Brazil")])

class PaginateDataGrid(DataGrid):
    template = "turbogears.widgets.templates.paginate_datagrid"

#TODO: Create PaginateDataGridDesc
#class PaginateDataGridDesc(CoreWD):



'''
@TODO: Laundry list of things yet to be done:
    * IdentityFilter should support HTTP Digest Auth
    * Also want to support Atom authentication (similar to digest)
'''
import md5
import sha
import threading

import cherrypy
import pkg_resources
import logging
log = logging.getLogger("turbogears.identity")

import turbogears
from turbogears.util import request_available
from turbogears.identity.exceptions import *

def create_default_provider():
    provider_plugin = turbogears.config.get( "identity.provider", "sqlobject" )
    plugins= pkg_resources.iter_entry_points( "turbogears.identity.provider",
                                              provider_plugin )

    log.debug( "Loading provider from plugin: %s", provider_plugin )

    for entrypoint in plugins:
        plugin= entrypoint.load()
        return plugin()

    raise IdentityConfigurationException( "IdentityProvider plugin missing: %s" %
                                          provider_plugin )


class IdentityWrapper(object):
    '''
    A wrapper class for the thread local data. This allows developers to access
    the current user information via turbogears.identity.current and get the
    identity for the current request.
    '''
    def identity(self):
        try:
            id= cherrypy.request.identity
        except AttributeError:
            id= None
        if not id:
            if not request_available():
                raise RequestRequiredException()
            raise IdentityManagementNotEnabledException()
        return id
        
    def __getattr__(self, name):
        '''
        return the named attribute of the global state
        '''
        identity= self.identity()
        if ("__str__"==name):
            return identity.__str__
        elif ("__repr__"==name):
            return identity.__repr__
        else:
            return getattr(identity, name)
        
    def __setattr__(self, name, value):
        '''
        stash a value in the global state
        '''
        identity= self.identity()
        setattr(identity,name,value)


class ProviderWrapper(object):
    
    def __getattr__(self, name):
        try:
            provider= cherrypy.request.identityProvider
        except AttributeError:
            try:
                provider = create_default_provider()
            except:
                provider= None
            
        if provider is None:
            if not request_available():
                raise RequestRequiredException()
            raise IdentityManagementNotEnabledException()
            
        return getattr(provider, name)

        
current= IdentityWrapper()
current_provider= ProviderWrapper()


def was_login_attempted():
    try:
        return cherrypy.request.identity_login_attempted
    except AttributeError:
        return False

                
def set_login_attempted( flag ):
    cherrypy.request.identity_login_attempted= flag
    
    
def set_current_identity(identity):
    cherrypy.request.identity = identity
    try:
        cherrypy.request.user_name = identity.user_name
    except AttributeError:
        cherrypy.request.user_name = None
    

def set_current_provider( provider ):
    cherrypy.request.identityProvider= provider

from turbogears.identity.conditions import *

def _encrypt_password(algorithm, password):
    """Hash the given password with the specified algorithm. Valid values
    for algorithm are 'md5' and 'sha1'. All other algorithm values will
    be essentially a no-op."""
    if isinstance(password, unicode):
        password_8bit = password.encode('UTF-8')
    else:
        password_8bit = password
    if "md5" == algorithm:
        hashed_password =  md5.new(password_8bit).hexdigest()
    elif "sha1" == algorithm:
        hashed_password = sha.new(password_8bit).hexdigest()
    else:
        hashed_password = password
    return hashed_password   

def encrypt_password(cleartext):
    # this next one ultimately needs to change to support SQLAlchemy
    return current_provider.encrypt_password(cleartext)

# declare what should be exported
__all__ = [ "IdentityManagementNotEnabledException",
            "IdentityConfigurationException",
            "IdentityFailure",
            "current", "current_provider",
            "set_current_identity", "set_current_provider",
            "set_identity_errors", "get_identity_errors",
            "was_login_attempted", "encrypt_password" ]



import sys
import os
import kid
import tempfile

from datetime import datetime

import turbogears
from turbogears.command.i18n import InternationalizationTool

work_dir = tempfile.mkdtemp()
locale_dir = os.path.join(work_dir, 'locale')
tool = None

turbogears.config.update({
    'i18n.locale_dir':locale_dir,
    'i18n.domain':'testmessages',
})

def setup(m):
    global tool
    tool = InternationalizationTool('0.9')

def teardown(m):
    import shutil
    shutil.rmtree(work_dir)

def test_creates_locale_dir():
    "Verify the locale directory got created as needed."
    assert not os.path.isdir(locale_dir)
    test_src_dir = os.path.join(work_dir, 'src')
    sys.argv = ['i18n.py', '--src-dir', test_src_dir, 'collect'] 
    tool.run()
    assert os.path.isdir(locale_dir), "locale directory not created"



"TurboGears Front-to-Back Web Framework"

import pkg_resources

from turbogears import config
from turbogears.controllers import expose, flash, validate, redirect, \
                                   error_handler, exception_handler, url
from turbogears import controllers, view, database, validators, command, \
                       i18n, widgets, startup, scheduler
from turbogears.release import version as __version__, author as __author__, \
                               email as __email__, license as __license__, \
                               copyright as __copyright__
from turbogears.widgets import mochikit
from turbogears.config import update_config
from turbogears.paginate import paginate

from turbogears.startup import start_server

extensions = pkg_resources.iter_entry_points("turbogears.extensions")
for entrypoint in extensions:
    ext = entrypoint.load()
    if hasattr(ext, "tgsymbols"):
        globals().update(ext.tgsymbols())

i18n.install() # adds _ (gettext) to builtins namespace


__all__ = ["url", "expose", "redirect", "validate", "flash",
           "error_handler", "exception_handler",
           "view", "controllers", "update_config",
           "database", "command", "validators", "mochikit", "widgets",
           "config", "start_server", "scheduler"]



import cherrypy
import turbogears.widgets as widgets
import turbogears.validators as validators

from turbogears.testutil import catch_validation_errors

class Request:
    validation_errors = {}

oldrequest = None

def setup_module():
    global oldrequest
    oldrequest = cherrypy.request
    cherrypy.request = Request()


def teardown_module():
    global oldrequest
    cherrypy.request = oldrequest




class SimpleSchema(validators.Schema):
    phone = validators.Int()


class NotSoSimpleSchema(SimpleSchema):
    def to_python(self, value, state=None):
        value = super(NotSoSimpleSchema, self).to_python(value, state)
        age = value.pop("age")
        phone = value.pop("phone")
        value["phoneage"] = age + phone
        return value


class SimpleFields(widgets.WidgetsList):
    age = widgets.TextField(validator=validators.Int())
    phone = widgets.TextField()


class SimpleForm(widgets.Form):
    fields = SimpleFields()
    validator = SimpleSchema()

class SimpleFieldSet(widgets.FieldSet):
    name = "fs"
    fields = SimpleFields()
    validator = SimpleSchema()





class TestSimpleForm:

    def test_simple_form(self):
        """ We can validate a simple form """

        form = SimpleForm()
        value = dict(age="22", phone="5555555")

        value = form.validate(value)
        print value
        assert value == dict(age=22, phone=5555555)

    def test_simple_form_bad(self):
        """ We can validate a simple form with errors """

        form = SimpleForm()
        value = dict(age="bad", phone="5555555")
        
        value, errors = catch_validation_errors(form, value)
        print value, errors
        assert errors.pop('age')
        assert not errors

    def test_simple_form_bad2(self):
        """ We can validate a simple form with errors, error from 
        the schema """

        form = SimpleForm()
        value = dict(age="22", phone="bad")

        value, errors = catch_validation_errors(form, value)
        print value, errors
        assert errors.pop('phone')
        assert not errors

    def test_not_so_simple_schema(self):
        """ we can validate a form with a custom schema """

        form = SimpleForm(validator=NotSoSimpleSchema())
        value = dict(age="22", phone="5555555")

        value = form.validate(value)
        print value
        assert value == dict(phoneage=5555577)

class TestNestedForm:
    
    def test_nested_form(self):
        """ We can validate a form with a nested fieldset """
        form = SimpleForm(fields=SimpleForm.fields + [SimpleFieldSet()])
        value = dict(age="22", phone="5555555", 
                     fs=dict(age="22", phone="5555555"))

        value = form.validate(value)
        print value
        assert value == dict(age=22, phone=5555555, 
                             fs=dict(age=22, phone=5555555))

    def test_nested_form_bad(self):
        """ We can validate a form with a nested fieldset with errors"""
        form = SimpleForm(fields=SimpleForm.fields + [SimpleFieldSet()])
        value = dict(age="22", phone="5555555", 
                     fs=dict(age="bad", phone="5555555"))

        value, errors = catch_validation_errors(form, value)
        print value, errors
        assert errors['fs'].pop('age')
        assert not errors.pop('fs')
        assert not errors
    
    def test_nested_form_bad2(self):
        """ We can validate a form with a nested fieldset with errors. error
        from the schema. """
        form = SimpleForm(fields=SimpleForm.fields + [SimpleFieldSet()])
        value = dict(age="22", phone="5555555", 
                     fs=dict(age="22", phone="bad"))

        value, errors = catch_validation_errors(form, value)
        print value, errors
        assert errors['fs'].pop('phone')
        assert not errors.pop('fs')
        assert not errors



#!/usr/bin/python
# -*- coding: utf-8 -*-

from turbogears import widgets, validators, controllers, expose, validate
from turbogears import config
from turbogears.testutil import catch_validation_errors, create_request
import cherrypy

class Request:
    input_values = {}
    validation_errors = {}

oldrequest = None

def setup_module():
    global oldrequest
    oldrequest = cherrypy.request
    cherrypy.request = Request()

def teardown_module():
    global oldrequest
    cherrypy.request = oldrequest

def test_rendering():
    """Forms can be rendered"""
    form = widgets.TableForm(fields=[widgets.TextField("name", label="Your Name")])
    output = form.render(action="mylocation", format="xhtml")
    print output
    assert "Your Name" in output
    assert 'name="name"' in output
    assert "submit" in output

def test_input_conversion():
    "Input for the whole form can be validated and converted"
    form = widgets.TableForm(fields=[widgets.TextField("name"),
                      widgets.TextField("age", validator=validators.Int())],
                      submit_text="Submit")
    values = dict(name="ed", age="15")
    values = form.validate(values)
    assert values["name"] == "ed"
    assert values["age"] == 15
    assert not values.has_key("submit")

def test_passing_instance():
    "You can pass an instance to a form for the value"
    form = widgets.TableForm(fields=[widgets.TextField("name"),
                      widgets.TextField("age", validator=validators.Int())],
                      submit_text="Submit")
    class Person(object):
        name = "ed"
        age = 892
    output = form.render(Person(), format="xhtml")
    print output
    assert 'value="ed"' in output
    assert 'value="892"' in output

def test_input_errors():
    "Data is stored in the request object if there are errors"
    form = widgets.TableForm(fields=[widgets.TextField("name"),
                      widgets.TextField("age", validator=validators.Int())])
    values = dict(name="ed", age="ed")
    values, errors = catch_validation_errors(form, values)
    print errors["age"]
    assert "enter an integer" in str(errors["age"])

class w1(widgets.FormField):
    javascript=[widgets.JSLink(__module__, "foo.js"),
                widgets.JSSource("alert('foo');"),
                widgets.JSSource("alert('foo again');",
                    widgets.js_location.bodybottom)]
    css=[widgets.CSSLink(__module__, "foo.css")]
    register=False

class w2(widgets.FormField):
    javascript=[widgets.JSLink(__module__, "foo.js")]
    css=[widgets.CSSLink(__module__, "foo.css")]
    register=False

def test_javascriptsets():
    "JavaScripts are only added once"
    form = widgets.TableForm(fields=[w1("foo"), w2("bar")])
    assert len(form.retrieve_javascript()) == 3

def test_csssets():
    "CSS references are added once"
    form = widgets.TableForm(fields=[w1("foo"), w2("bar")])
    assert len(form.retrieve_css()) == 1

def test_creation():
    class TestFormFields(widgets.WidgetsList):
        foo = widgets.TextField()
        bar = widgets.CheckBox()
    t = widgets.TableForm(fields=TestFormFields() + [widgets.TextField('a')])
    wlist = t.fields
    assert len(wlist) == 3, '%s' % [x.name for x in wlist]
    assert wlist[0].name == 'foo'
    assert wlist[1].name == 'bar'
    assert wlist[2].name == 'a'

def test_creation_overriding():
    class TestFormFields(widgets.WidgetsList):
        foo = widgets.TextField()
        bar = widgets.CheckBox()
    fields = TestFormFields()
    fields[1] = widgets.TextField('bar')
    t = widgets.TableForm(fields=fields)
    assert len(t.fields) == 2, '%s' % [x.name for x in t.fields]

def test_disabled_widget():
    form = widgets.TableForm(fields=[widgets.TextField("name"),
                      widgets.TextField("age", validator=validators.Int())],
                      submit_text="Submit")
    output = form.render(disabled_fields=["age"])
    print output
    assert "age" not in output

def test_class_attributes_form():
    class TestForm(widgets.ListForm):
        fields =  [widgets.TextField("name"), widgets.TextField("age")]
        validator = validators.Schema()
        submit_text = "gimme"

    form = TestForm()
    output = form.render()
    assert "name" in output
    assert "age" in output
    assert "gimme" in output

    form = TestForm(fields = TestForm.fields + [widgets.TextField("phone")],
                    submit_text = "your number too")
    output = form.render()
    assert "phone" in output
    assert "your number too" in output

def test_help_text():
    class TestForm(widgets.ListForm):
        fields =  [
            widgets.TextField("name", help_text="Enter your name here"),
            widgets.TextField("age", help_text="Enter your age here"),
        ]
    form = TestForm()
    output = form.render()
    assert "age here" in output
    assert "name here" in output

class CallableCounter:
    def __init__(self):
        self.counter = 0
    def __call__(self):
        self.counter += 1
        return [(1, 'foobar')]

def test_callable_options_for_selection_field():
    cc = CallableCounter()
    w = widgets.CheckBoxList('collections',
            label='Collections',
            options=cc)
    assert cc.counter == 1 # called once to guess validator
    cc = CallableCounter()
    w = widgets.CheckBoxList('collections',
            label='Collections',
            validator=validators.Int(),
            options=cc)
    assert cc.counter == 0 # cc shouldn't be called if validator is provided

nestedform = widgets.TableForm(fields=[
        widgets.FieldSet("foo", fields=[
            widgets.TextField("name"),
            widgets.TextField("age")
        ])
    ])
class NestedController(controllers.Controller):
    def checkform(self, foo):
        self.foo = foo
    checkform = expose()(checkform)
    checkform = validate(form=nestedform)(checkform)

def test_nested_variables():
    cherrypy.request = oldrequest
    newroot = NestedController()
    cherrypy.root = None
    cherrypy.tree.mount_points = {}
    cherrypy.tree.mount(newroot, "/")
    url = u"/checkform?foo.name=Kevin&foo.age=some%20Numero".encode("utf-8")
    create_request(url)
    assert config.get("decoding_filter.encoding", path="/") == "utf8"
    assert newroot.foo
    assert newroot.foo["name"] == "Kevin"
    assert newroot.foo["age"] == u"some Numero"

def test_field_for():
    cherrypy.request.validation_errors = dict(foo=dict(foo='error'))
    template = """\
    <div xmlns:py="http://purl.org/kid/ns#">
        ${field_for('foo').fq_name}.appears
        ${field_for('foo').error}_appears
        ${field_for('foo').field_id}_appears
        ${field_for('foo').display(value_for('foo'), **params_for('foo'))}
    </div>
    """
    textfield = widgets.TextField("foo")
    fieldset = widgets.FieldSet("foo", fields=[textfield], template=template)
    form = widgets.Form("form", fields=[fieldset], template=template)
    # Good example below of how you can pass parameters and values to nested
    # widgets.
    value = dict(foo=dict(foo="HiYo!"))
    params = dict(attrs=dict(foo=dict(foo=dict(size=100))))
    params['format'] = 'xhtml'
    output = form.render(value, **params)
    assert "form_foo_appears" in output
    assert "form_foo_foo_appears" in output
    assert "foo.appears" in output
    assert "foo.foo.appears" in output
    assert "error_appears" in output
    assert "textfield" in output
    assert "HiYo!" in output
    assert "size=\"100\"" in output



from datetime import timedelta, datetime

from sqlobject import *
from sqlobject.sqlbuilder import *

from turbogears.visit.api import BaseVisitManager, Visit
from turbogears.database import PackageHub
from turbogears import config
from turbogears.util import load_class

hub = PackageHub("turbogears.visit")
__connection__ = hub

import logging

log = logging.getLogger("turbogears.visit.sovisit")

visit_class = None

class SqlObjectVisitManager(BaseVisitManager):
    def __init__(self, timeout):
        global visit_class
        visit_class_path = config.get("visit.soprovider.model", 
            "turbogears.visit.sovisit.TG_Visit")
        visit_class = load_class(visit_class_path)
        if visit_class:
            log.info("Succesfully loaded \"%s\"" % visit_class_path)
        super(SqlObjectVisitManager,self).__init__( timeout )

    def create_model(self):
        try:
            # Create the Visit table if it doesn't already exist (idea from CatWalk)
            hub.begin()
            visit_class.createTable(ifNotExists=True)
            hub.commit()
            hub.end()
        except KeyError:
            # No database configured...
            log.info( "No database is configured: Visit Tracking is disabled." )
            return

    def new_visit_with_key(self, visit_key):
        hub.begin()
        visit= visit_class( visit_key=visit_key, expiry=datetime.now()+self.timeout )
        hub.commit()
        hub.end()
        return Visit( visit_key, True )

    def visit_for_key(self, visit_key):
        '''
        Return the visit for this key or None if the visit doesn't exist or has
        expired.
        '''
        visit= visit_class.lookup_visit( visit_key )
        now= datetime.now()
        if not visit or visit.expiry < now:
            return None
        # Visit hasn't expired, extend it
        self.update_visit( visit_key, now+self.timeout )
        return Visit( visit_key, False )

    def update_queued_visits(self, queue):
        if hub is None: # if VisitManager extension wasn't shutted down cleanly
            return
        hub.begin()
        try:
            conn= hub.getConnection()
            try:
                # Now update each of the visits with the most recent expiry
                for visit_key,expiry in queue.items():
                    u= Update( visit_class.q, {visit_class.q.expiry.fieldName:expiry},
                               where=(visit_class.q.visit_key==visit_key) )
                    conn.query( conn.sqlrepr(u) )
                hub.commit()
            except:
                hub.rollback()
                raise
        finally:
            hub.end()

class TG_Visit(SQLObject):
    class sqlmeta:
        table="tg_visit"

    visit_key= StringCol( length=40, alternateID=True,
                          alternateMethodName="by_visit_key" )
    created= DateTimeCol( default=datetime.now )
    expiry= DateTimeCol()

    def lookup_visit( cls, visit_key ):
        try:
            return cls.by_visit_key( visit_key )
        except SQLObjectNotFound:
            return None
    lookup_visit= classmethod(lookup_visit)



# Formatting configuration for locale om_KE

languages={'om': 'Oromoo'}

countries={'ET': 'Itoophiyaa', 'KE': 'Keeniyaa'}

months=['Amajjii', 'Guraandhala', 'Bitooteessa', 'Elba', 'Caamsa', 'Waxabajjii', 'Adooleessa', 'Hagayya', 'Fuulbana', 'Onkololeessa', 'Sadaasa', 'Muddee']

abbrMonths=['Ama', 'Gur', 'Bit', 'Elb', 'Cam', 'Wax', 'Ado', 'Hag', 'Ful', 'Onk', 'Sad', 'Mud']

days=['Wiixata', 'Qibxata', 'Roobii', 'Kamiisa', 'Jimaata', 'Sanbata', 'Dilbata']

abbrDays=['Wix', 'Qib', 'Rob', 'Kam', 'Jim', 'San', 'Dil']

dateFormats={'medium': '%d-%%(abbrmonthname)s-%y', 'full': '%%(dayname)s, %%(monthname)s %d, %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}


"""Convenient validators and converters for data coming in from the web.

This module also imports everything from formencode.validators, so all
common validation routines are available here."""

import pkg_resources
#XXX Remove in 1.0.3 when everyone has already upgraded FE
#    so we don't need to keep this in sync with setup.py
pkg_resources.require("FormEncode >= 0.7.1")

import time
import re
from datetime import datetime
import cgi # FieldStorageUploadConverter
import warnings

import simplejson
from formencode.validators import *
from formencode.compound import *
from formencode.api import Invalid, NoDefault
from formencode.schema import Schema
from formencode import ForEach

from turbogears.i18n import format
from turbogears import util
from turbojson import jsonify

from formencode import validators # Needed to disambiguate the Number validator... 

import __builtin__

def _(s): return s # dummy

Validator.gettextargs['domain'] = 'FormEncode' # FormEncode should call Tg's gettext \
                                               # function with domain = "FormEncode"

class TgFancyValidator(FancyValidator):
    gettextargs = {'domain':'TurboGears'}
    
class Money(TgFancyValidator):
    
    messages = {
        'badFormat': _('Invalid number format'),
        'empty': _('Empty values not allowed'),
    }
    
    def __init__(self, allow_empty=None, *args, **kw):
        if allow_empty is not None:
            warnings.warn("Use not_empty instead of allow_empty",
                          DeprecationWarning, 2)
            not_empty = not allow_empty
            kw["not_empty"] = not_empty
        super(Money, self).__init__(*args, **kw)

    def _to_python(self, value, state):
        """ parse a string and returns a float or integer """
        try:
            return format.parse_decimal(value)
        except ValueError:
            raise Invalid(self.message('badFormat', state), value, state)

    def _from_python(self, value, state):
        """ returns a string using the correct grouping """
        return format.format_currency(value)


class Number(TgFancyValidator):

    def _to_python(self, value, state):
        """ parse a string and returns a float or integer """
        if isinstance(value, basestring):
            try:
                value = format.parse_decimal(value)
            except ValueError:
                pass
        return validators.Number.to_python(value, state)

    def _from_python(self, value, state):
        """ returns a string using the correct grouping """
        dec_places = util.find_precision(value)
        if dec_places > 0:
            return format.format_decimal(value, dec_places)
        else:
            return format.format_number(value)


class DateTimeConverter(TgFancyValidator):

    """
    Converts Python date and datetime objects into string representation and back.
    """
    messages = {
        'badFormat': _('Invalid datetime format'),
        'empty': _('Empty values not allowed'),
    }

    def __init__(self, format = "%Y/%m/%d %H:%M", allow_empty = None,
                *args, **kwargs):
        if allow_empty is not None:
            warnings.warn("Use not_empty instead of allow_empty",
                          DeprecationWarning, 2)
            not_empty = not allow_empty
            kwargs["not_empty"] = not_empty
        super(TgFancyValidator, self).__init__(*args, **kwargs)
        self.format = format

    def _to_python(self, value, state):
        """ parse a string and return a datetime object. """
        if value and isinstance(value, datetime):
            return value
        else:
            try:
                tpl = time.strptime(value, self.format)
            except ValueError:
                raise Invalid(self.message('badFormat', state), value, state)
            # shoudn't use time.mktime() because it can give OverflowError,
            # depending on the date (e.g. pre 1970) and underlying C library
            return datetime(year=tpl.tm_year, month=tpl.tm_mon, day=tpl.tm_mday,
                    hour=tpl.tm_hour, minute=tpl.tm_min, second=tpl.tm_sec)

    def _from_python(self, value, state):
        if not value:
            return None
        elif isinstance(value, datetime):
            # Python stdlib can only handle dates with year greater than 1900
            if value.year <= 1900:
                return strftime_before1900(value, self.format)
            else:
                return value.strftime(self.format)
        else:
            return value

# formencode trunk contains UnicodeString implementation 
# but it is different from ours and was broken at the time.
# remove this impl. when formencode.validators.UnicodeString will be identical to ours.
class UnicodeString(String):
    encoding = 'utf-8'
    messages = {
        'badEncoding' : _("Invalid data or incorrect encoding"),
    }
    def __init__(self, inputEncoding=None, outputEncoding=None, **kw):
        String.__init__(self, **kw)
        self.inputEncoding = inputEncoding or self.encoding
        self.outputEncoding = outputEncoding or self.encoding
    def _to_python(self, value, state):
        if value:
            if isinstance(value, unicode):
                return value
            if hasattr(value, '__unicode__'):
                return unicode(value)
            try:
                return unicode(value, self.inputEncoding)
            except UnicodeDecodeError:
                raise Invalid(self.message('badEncoding', state), value, state)
        return u''
    def _from_python(self, value, state):
        if hasattr(value, '__unicode__'):
            value = unicode(value)
        if isinstance(value, unicode):
            return value.encode(self.outputEncoding)
        return str(value)

# another formencode workaround,
# see #1464357 on FE bugtracker (http://tinyurl.com/lm9ae).
# Custom version of FieldStorage validator that does not break FE schema validator.
class FieldStorageUploadConverter(TgFancyValidator):
    def to_python(self, value, state=None):
        if isinstance(value, cgi.FieldStorage):
            if value.filename:
                return value
            raise Invalid('invalid', value, state)
        else:
            return value 

# For translated messages that are not wrapped in a Validator.messages
# dictionary, we need to reinstate the Turbogears gettext function under
# the name "_", with the "TurboGears" domain, so that the TurboGears.mo
# file is selected.
import turbogears.i18n
_ = lambda s: turbogears.i18n.gettext(s, domain='TurboGears')

class MultipleSelection(ForEach):
    if_missing = NoDefault
    if_empty = []

    def to_python(self, value, state=None):
        try:
            return super(MultipleSelection, self).to_python(value, state)
        except Invalid:
            raise Invalid(_("Please select at least a value"), value, state)

class Schema(Schema):
    """ A Schema validator """
    filter_extra_fields = True
    allow_extra_fields = True
    if_key_missing = None

    def from_python(self,value,state=None):
        # The Schema shouldn't do any from_python conversion because
        # adjust_value already takes care of that for all childs.
        return value

class JSONValidator(TgFancyValidator):

    def _from_python(self, value, state):
        return jsonify.encode(value)

    def _to_python(self, value, state):
        return simplejson.loads(value)

_illegal_s = re.compile(r"((^|[^%])(%%)*%s)")

def _findall(text, substr):
     # Also finds overlaps
     sites = []
     i = 0
     while 1:
         j = text.find(substr, i)
         if j == -1:
             break
         sites.append(j)
         i = j+1
     return sites

def strftime_before1900(dt, fmt):
    """
    A strftime implementation that supports proleptic Gregorian dates before 1900.

    @see: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/306860
    """
    import datetime
    if _illegal_s.search(fmt):
        raise TypeError("This strftime implementation does not handle %s")
    if dt.year > 1900:
        return dt.strftime(fmt)

    year = dt.year
    # For every non-leap year century, advance by
    # 6 years to get into the 28-year repeat cycle
    delta = 2000 - year
    off = 6*(delta // 100 + delta // 400)
    year = year + off

    # Move to around the year 2000
    year = year + ((2000 - year)//28)*28
    timetuple = dt.timetuple()
    s1 = time.strftime(fmt, (year,) + timetuple[1:])
    sites1 = _findall(s1, str(year))

    s2 = time.strftime(fmt, (year+28,) + timetuple[1:])
    sites2 = _findall(s2, str(year+28))

    sites = []
    for site in sites1:
        if site in sites2:
            sites.append(site)

    s = s1
    syear = "%4d" % (dt.year,)
    for site in sites:
        s = s[:site] + syear + s[site+4:]
    return s

